#include "Cell.h"

//temporary for testing I think
#define Y_TILE_COUNT 12
#define X_TILE_COUNT 20
#define MAPC(x,y) mapCells[x*Y_TILE_COUNT + y]

//tile properties
//00 none
//01 broken
//10 invisible
//11 death
#define PROPERTY_NONE 0
#define PROPERTY_BROKEN 1
#define PROPERTY_INVISIBLE 2
#define PROPERTY_DEATH 3

//define mapCells
Cell mapCells[240];

#define NO_TILE 255

void initAllCells()
{
    u8 i, j;
    u16 position = 0; //if the grid size remains 240 we could use u8

    for (i = 0; i < X_TILE_COUNT; ++i)
    {            
        for (j = 0; j < Y_TILE_COUNT; ++j)
        {
            mapCells[position].graphicType = GRAPHIC_TYPE_EMPTY;
            mapCells[position].broken = 0;
            mapCells[position].used = 0;
            mapCells[position].visible = 1;
            mapCells[position].death = 0;
            mapCells[position].solid = 0;
            
            ++position;
        }
    }
}

void initTileCells(u8* mapArray)
{
    u8 i, j;
    u16 position = 0; //if the grid size remains 240 we could use u8

    for (i = 0; i < X_TILE_COUNT; ++i)
    {            
        for (j = 0; j < Y_TILE_COUNT; ++j)
        {
			u8 graphic = mapArray[position];

            if(graphic != NO_TILE)
            {
                //extract the properties from the mapArray to load into the cell
                if((graphic & PROPERTY_DEATH) == PROPERTY_DEATH)
                {
                    mapCells[position].death = 1;
                }
                else if(graphic & PROPERTY_BROKEN)
                {
                    mapCells[position].broken = 1;
                }
                else if(graphic & PROPERTY_INVISIBLE)
                {
                    mapCells[position].visible = 0;
                }
				
				//shift the array two positions to the right to remove the properties now that we have extracted them
				graphic >>= 2;
				
                mapCells[position].graphicNumber.tile = graphic;
                mapCells[position].graphicType = GRAPHIC_TYPE_TILE;
                mapCells[position].solid = 1;

				//set a cell to non solid if it is invisible or death
				if (mapCells[position].visible == 0 || mapCells[position].death)
				{
                    mapCells[position].solid = 0;
				}
            }
            
            ++position;
        }
    }
}

void initItemCells(u8* rawMapItems, u8 itemCount, u8 itemComponentCount)
{
    u8 i = 0;
    u8 x = 0;
    u8 y = 0;

    for(i = 0; i<itemCount*itemComponentCount; i+=itemComponentCount)
    {	
		x = rawMapItems[i];
        y = rawMapItems[i+1];
		MAPC(x, y).graphicNumber.item = rawMapItems[i+2];
        MAPC(x, y).graphicType = GRAPHIC_TYPE_ITEM;
        
        //this is the index in the mapItems array
        MAPC(x, y).index = i / itemComponentCount;
    }
}

void initInteractableCells(u8* rawMapInteractables, u8 interactableCount, u8 interactableComponentCount)
{
    u8 i = 0;
    u8 x = 0;
    u8 y = 0;

    for(i = 0; i<interactableCount*interactableComponentCount; i+=interactableComponentCount)
    {
        x = rawMapInteractables[i];
        y = rawMapInteractables[i+1];
		MAPC(x, y).graphicNumber.item = rawMapInteractables[i+2];
        MAPC(x, y).graphicType = GRAPHIC_TYPE_INTERACTABLE;
		MAPC(x, y).broken = rawMapInteractables[i+5];
        
        //this is the index in the mapInteractables array
        MAPC(x, y).index = i / interactableComponentCount;
    }
}

void clearCell(u8 x, u8 y)
{
    u8* memory = cpct_getScreenPtr(CPCT_VMEM_START, xPositions[x], yPositions[y]);
    cpct_drawSolidBox(memory, backgroundColour, TILE_WIDTH_BYTES, TILE_HEIGHT);
}

void redrawCellOfType(u8 x, u8 y, u8 type)
{
    u8* memory = cpct_getScreenPtr(CPCT_VMEM_START, xPositions[x], yPositions[y]);
    
    if(type == GRAPHIC_TYPE_TILE)
    {
        cpct_drawSprite(tileGraphicSet[MAPC(x, y).graphicNumber.tile], memory, TILE_WIDTH_BYTES, TILE_HEIGHT);
    }   
    else if(type == GRAPHIC_TYPE_INTERACTABLE)
    {
        cpct_drawSprite(interactableGraphicSet[MAPC(x, y).graphicNumber.interactable], memory, TILE_WIDTH_BYTES, TILE_HEIGHT);

		//draw an x if the interactable is in a broken state 
		if (MAPC(x, y).broken)
		{
			cpct_drawSpriteMasked(G_broken_interactable_indicator, memory, TILE_WIDTH_BYTES, TILE_HEIGHT);
		}
    }
    else if(type == GRAPHIC_TYPE_ITEM)
    {
        cpct_drawSprite(itemGraphicSet[MAPC(x, y).graphicNumber.item], memory, TILE_WIDTH_BYTES, TILE_HEIGHT);
    }
}

void redrawCell(u8 x, u8 y)
{
    u8* memory = cpct_getScreenPtr(CPCT_VMEM_START, xPositions[x], yPositions[y]);
    u8 type = MAPC(x,y).graphicType;
    
	//only draw a cell if it is visible
	if (MAPC(x, y).visible)
	{
		if (type == GRAPHIC_TYPE_TILE)
		{
            cpct_drawSprite(tileGraphicSet[MAPC(x, y).graphicNumber.tile], memory, TILE_WIDTH_BYTES, TILE_HEIGHT);
            
            //draw the cracked tile indicator if it is broken
			if(MAPC(x, y).broken)
            {
                cpct_drawSpriteMasked (G_cracked_tile_indicator, memory, TILE_WIDTH_BYTES, TILE_HEIGHT);
            }
		}
		else if (type == GRAPHIC_TYPE_INTERACTABLE)
		{
			cpct_drawSprite(interactableGraphicSet[MAPC(x, y).graphicNumber.interactable], memory, TILE_WIDTH_BYTES, TILE_HEIGHT);

			//draw an x if the interactable is in a broken state 
			if (MAPC(x, y).broken)
			{
				cpct_drawSpriteMasked(G_broken_interactable_indicator, memory, TILE_WIDTH_BYTES, TILE_HEIGHT);
			}
		}
		else if (type == GRAPHIC_TYPE_ITEM)
		{
			cpct_drawSprite(itemGraphicSet[MAPC(x, y).graphicNumber.item], memory, TILE_WIDTH_BYTES, TILE_HEIGHT);
		}
	}
}