#ifndef CELL_H
#define CELL_H

#include <cpctelera.h>
#include "GameTypes.h"
#include "sprites/EnemySprites.h"
#include "sprites/InteractableSprites.h"
#include "sprites/ItemSprites.h"
#include "sprites/TileSprites.h"
#include "sprites/IndicatorSprites.h"
#include "Mediator.h"

#define GRAPHIC_TYPE_EMPTY 0
#define GRAPHIC_TYPE_TILE 1
#define GRAPHIC_TYPE_ITEM 2
#define GRAPHIC_TYPE_INTERACTABLE 3

static const u8 xPositions[20] = { 0, 4, 8, 12, 16, 20, 24, 28, 32, 36, 40, 44, 48, 52, 56, 60, 64, 68, 72, 76 }; 
static const u8 yPositions[12] = { 0, 16, 32, 48, 64, 80, 96, 112, 128, 144, 160, 176 };

typedef union GraphicNumber GraphicNumber;
typedef struct Cell Cell;

union GraphicNumber
{
    u8 tile;
    u8 item;
    u8 interactable;
};

struct Cell
{
    GraphicNumber graphicNumber;
    u8 graphicType : 2; //0 empty, 1 tile, 2 item, 3 interactable
    u8 visible : 1;
    u8 broken : 1;
    u8 used : 1;
    u8 death : 1;
    u8 solid : 1;
    u8 index : 4;
};

//240 cells in a 20x12 grid
extern Cell mapCells[240];

void initTileCells(u8* mapArray);

void initItemCells(u8* rawMapItems, u8 itemCount, u8 itemComponentCount);

void initInteractableCells(u8* rawMapInteractables, u8 interactableCount, u8 interactableComponentCount);

//clear the square on the screen to blank (eg pickup an item)
void clearCell(u8 x, u8 y);

//redraw the content of the square on the screen (eg state change)
void redrawCellOfType(u8 x, u8 y, u8 type);

void redrawCell(u8 x, u8 y);

#endif