#include "Enemy.h"
#define MAPA(x,y) mapArray[x*SCREEN_TILES_Y + y]
#define MAPB(x,y) x*SCREEN_TILES_Y + y

//the tiles that the enemy currently occupies
u8 enemyOccupiedXTileLeft_ = 0;
u8 enemyOccupiedXTileRight_ = 0;
u8 enemyOccupiedYTileTop_ = 0;
u8 enemyOccupiedYTileBottom_ = 0;

void drawEnemy(Enemy* enemy)
{
	u8* memory;

	//draw over the previous tail
	if ((enemy->type == ENEMY_TYPE_LEFT_RIGHT || enemy->type == ENEMY_TYPE_UP_DOWN || enemy->type == ENEMY_TYPE_UP_DOWN2 || enemy->type == ENEMY_TYPE_LEFT_RIGHT2) 
	&& (enemy->previousPosition.x != enemy->object.position.x || enemy->previousPosition.y != enemy->object.position.y))
	{
		//cover the previous position
		memory = cpct_getScreenPtr(CPCT_VMEM_START, enemy->previousPosition.x, enemy->previousPosition.y);
		cpct_drawSolidBox(memory, backgroundColour, enemy->object.width, enemy->object.height);
	}

    //draw the specified enemy in the enemy graphic set
	memory = cpct_getScreenPtr(CPCT_VMEM_START, enemy->object.position.x, enemy->object.position.y);

    //todo handle types and graphics better for drawing enemies
    cpct_drawSprite(enemyGraphicSet[enemy->type], memory, TILE_WIDTH_BYTES, TILE_HEIGHT);

	//only droppers and lasers have projectiles
	if (enemy->state == ENEMY_STATE_ACTIVE && (enemy->type == ENEMY_TYPE_DOWN_DROPPER || enemy->type == ENEMY_TYPE_DOWN_LASER))
	{
		drawProjectile(&enemy->projectile);
	}
}

void logicEnemy(Enemy* enemy)
{
	//projectile only needs to be modified when active
	if(enemy->state == ENEMY_STATE_ACTIVE)
	{
		logicProjectile(&enemy->projectile);

		//using enemy timer count to slow the enemy movement down
		if(enemyTimerCount > 90 && (enemy->type == ENEMY_TYPE_LEFT_RIGHT || enemy->type == ENEMY_TYPE_LEFT_RIGHT2))
		{
			u8 belowOccupiedYTileBottom = 0;

			calculateEnemyOccupiedXGridPositions(enemy->object.position.x, enemy->object.velocity.x, enemy->object.width);
			calculateEnemyOccupiedYGridPositions(enemy->object.position.y, enemy->object.velocity.y, enemy->object.height);

			belowOccupiedYTileBottom = enemyOccupiedYTileBottom_ + 1;

			//save previous position
			enemy->previousPosition.x = enemy->object.position.x;

			if(enemy->direction == ENEMY_LEFT)
			{
				//check to see if the cell to the left is clear AND
				//check to see that the tile to the left and below is solid
				if (mapCells[MAPB(enemyOccupiedXTileLeft_, enemyOccupiedYTileTop_)].graphicType == GRAPHIC_TYPE_EMPTY && 
					mapCells[MAPB(enemyOccupiedXTileLeft_, belowOccupiedYTileBottom)].graphicType == GRAPHIC_TYPE_TILE)
				{
					enemy->object.position.x += enemy->object.velocity.x;
				}
				else
				{
					//change direction
					enemy->object.velocity.x *= -1;
					enemy->direction *= -1;
				}
			}			
			else
			{
				//check to see if the cell to the right is clear AND
				//check to see that there is a tile to the right and below is solid
				if (mapCells[MAPB(enemyOccupiedXTileRight_, enemyOccupiedYTileTop_)].graphicType == GRAPHIC_TYPE_EMPTY &&
					mapCells[MAPB(enemyOccupiedXTileRight_, belowOccupiedYTileBottom)].graphicType == GRAPHIC_TYPE_TILE)
				{
					enemy->object.position.x += enemy->object.velocity.x;
				}
				else
				{
					//change direction
					enemy->object.velocity.x *= -1;
					enemy->direction *= -1;
				}
			}
			
		}
		else if(enemyTimerCount > 45 && enemy->type == ENEMY_TYPE_UP_DOWN || enemy->type == ENEMY_TYPE_UP_DOWN2)
		{
			calculateEnemyOccupiedXGridPositions(enemy->object.position.x, enemy->object.velocity.x, enemy->object.width);
			calculateEnemyOccupiedYGridPositions(enemy->object.position.y, enemy->object.velocity.y, enemy->object.height);

			//save previous position
			enemy->previousPosition.y = enemy->object.position.y;

			if(enemy->direction == ENEMY_UP)
			{
				//check to see if the cell above is clear when going up
				if (mapCells[MAPB(enemyOccupiedXTileLeft_, enemyOccupiedYTileTop_)].graphicType == GRAPHIC_TYPE_EMPTY)
				{
					enemy->object.position.y += enemy->object.velocity.y;
				}
				else
				{
					//change direction
					enemy->object.velocity.y *= -1;
					enemy->direction *= -1;
				}
			}
			else
			{
				//cehck to see if the cell below is clear when going down
				if (mapCells[MAPB(enemyOccupiedXTileLeft_, enemyOccupiedYTileBottom_)].graphicType == GRAPHIC_TYPE_EMPTY)
				{
					enemy->object.position.y += enemy->object.velocity.y;
				}
				else
				{
					//change direction
					enemy->object.velocity.y *= -1;
					enemy->direction *= -1;
				}
			}			
		}		
	}
}

void activateEnemy(Enemy* enemy)
{
	if(enemy->state == ENEMY_STATE_ACTIVE)
	{
		enemy->state = ENEMY_STATE_INACTIVE;

		//remove the projectile when enemy state changed to off
		enemy->projectile.exists = FALSE;

		//clear the projectile from the screen as it no longer exists
		clearProjectile(&enemy->projectile);
	}
	else
	{
		enemy->state = ENEMY_STATE_ACTIVE;

		if (enemy->projectile.exists == FALSE)
		{
			//reinitalise the projectile if it becomes active again
			initProjectile(&enemy->projectile, &enemy->object, enemy->projectile.object.width, enemy->projectile.object.height, enemy->projectile.end, (enemy->type == ENEMY_TYPE_DOWN_DROPPER));
		}
	}
}

//todo change to bitshift >> 2to improve performance
void calculateEnemyOccupiedXGridPositions(u8 xPosition, i8 xVelocity, u8 width)
{	
	enemyOccupiedXTileLeft_ = (xPosition + xVelocity) / TILE_WIDTH_BYTES;

	//the -1 is because when you add the width we end up going over which gives us a different tile to what we want
	enemyOccupiedXTileRight_ = (xPosition + width + xVelocity - 1) / TILE_WIDTH_BYTES;
}

//todo change to bitshift >> 2to improve performance
void calculateEnemyOccupiedYGridPositions(u8 yPosition, i8 yVelocity, u8 height)
{
	enemyOccupiedYTileTop_ = (yPosition + yVelocity) / TILE_HEIGHT;

	//the -1 is because when you add the height we end up going over which gives us a different tile to what we want
	enemyOccupiedYTileBottom_ = (yPosition + height + yVelocity - 1) / TILE_HEIGHT;
}