#include "Fitzroy.h"

//33 is the minimum
const u8 GRAVITY = 100;
#define FALL_GRAVITY 120
#define MAPA(x,y) mapArray[x*SCREEN_TILES_Y + y]
#define MAPB(x,y) x*SCREEN_TILES_Y + y
#define LIFE_COUNT 3

#define MAX_VERTICAL_VELOCITY -2000
//-1000 is normal
//-1550 would be a big jump
#define SHOES_JUMP_VELOCITY -1270
#define INITIAL_JUMP_VELOCITY -1000//-1270 //-1000//-1550// -1000
#define TERMINAL_VELOCITY 2000//3000

#define FITZROY_WIDTH_ 2
#define FITZROY_HEIGHT_ 16

#define FITZROY_RIGHT 1
#define FITZROY_LEFT -1

Fitzroy fitzroy;

u8 tailColour_ = 0;

//the tiles that the player currently occupies
u8 occupiedXTileLeft_ = 0;
u8 occupiedXTileRight_ = 0;
u8 occupiedYTileTop_ = 0;
u8 occupiedYTileBottom_ = 0;

void initFitzroy(Fitzroy* fitzroy)
{
	//reset fitzroy direction if it is left from the previous level
	if (fitzroy->direction == FITZROY_LEFT)
	{
		cpct_hflipSpriteM0(fitzroy->object.width, fitzroy->object.height, G_fitzroy);
	}

	//add 200 points for perfect level score
	if(fitzroy->lifeCount == LIFE_COUNT)
	{
		gameHud.gameScore += 200;
	}

	//set some initial values for fitzroy
	fitzroy->object.height = FITZROY_HEIGHT_;
	fitzroy->object.width = FITZROY_WIDTH_;
	fitzroy->object.position.x = mapFitzroyXPosition * TILE_WIDTH_BYTES;
	fitzroy->object.position.y = mapFitzroyYPosition * TILE_HEIGHT;
	fitzroy->previousPosition.x = fitzroy->object.position.x;
	fitzroy->previousPosition.y = fitzroy->object.position.y;

	fitzroy->physicsXPosition = fitzroy->object.position.x * 255;
	fitzroy->physicsYPosition = fitzroy->object.position.y * 255;
	fitzroy->physicsXVelocity = 0;
	fitzroy->physicsYVelocity = 0;
	fitzroy->lifeCount = LIFE_COUNT;
	fitzroy->changedDirection = FALSE;
	fitzroy->state = WALKING;
	fitzroy->direction = FITZROY_RIGHT;

	if(hiddenFlag & DRAW_TAILS)
	{
		tailColour_ = cpct_px2byteM0(8,8);
	}
	else
	{
		//default is background colour which is effectively no tail drawn
		tailColour_ = backgroundColour;	
	}
}

void drawFitzroy(Fitzroy* fitzroy)
{
	u8* memory;

	//draw over the previous tail
	if (fitzroy->previousPosition.x != fitzroy->object.position.x || fitzroy->previousPosition.y != fitzroy->object.position.y)
	{
		//cover the previous position
		memory = cpct_getScreenPtr(CPCT_VMEM_START, fitzroy->previousPosition.x, fitzroy->previousPosition.y);
		cpct_drawSolidBox(memory, tailColour_, fitzroy->object.width, fitzroy->object.height);

		calculateOccupiedXGridPositions(fitzroy->previousPosition.x, 0, fitzroy->object.width);
		calculateOccupiedYGridPositions(fitzroy->previousPosition.y, 0, fitzroy->object.width);
		redrawPreviouslyOccupiedCells();
	}
	
	memory = cpct_getScreenPtr(CPCT_VMEM_START, fitzroy->object.position.x, fitzroy->object.position.y);

	if(fitzroy->state != FLYING)
	{
		if(fitzroy->changedDirection)
		{
			cpct_hflipSpriteM0(fitzroy->object.width, fitzroy->object.height, G_fitzroy);	
			
			fitzroy->changedDirection = FALSE;
		}

		//draw fitzroy in the normal position		
		cpct_drawSprite(G_fitzroy, memory, fitzroy->object.width, fitzroy->object.height);
	}
	else
	{
		if(fitzroy->changedDirection)
		{
			cpct_hflipSpriteM0(fitzroy->object.width, fitzroy->object.height, G_fitzroy_chair);

			fitzroy->changedDirection = FALSE;
		}

		//draw fitzroy flying
		cpct_drawSprite(G_fitzroy_chair, memory, fitzroy->object.width, fitzroy->object.height);
	}
}

void logicFitzroy(Fitzroy* fitzroy, Keys* keys)
{
	//temporarily variable to test that a position is valid
	u8 temporaryPosition = 0;

	u8 i = 0;

	//moved this to the top that we don't test for collision with enemies until after the changes have been drawn
	//enemies only cause dmanage while invincibility is not active
	if (invincibilityActive == FALSE)
	{
		//check for enemy presence
		for (i = 0; i<mapEnemyCount; ++i)
		{
			//only active enemies can cause damange
			if (mapEnemies[i].state == ENEMY_STATE_ACTIVE)
			{
				//right of fitzroy > left of enemy
				//left of fitzroy <
				if (fitzroy->object.position.x + fitzroy->object.width >(mapEnemies[i].object.position.x) &&
					fitzroy->object.position.x < (mapEnemies[i].object.position.x + mapEnemies[i].object.width))
				{
					if (fitzroy->object.position.y + fitzroy->object.height >(mapEnemies[i].object.position.y) &&
						fitzroy->object.position.y < (mapEnemies[i].object.position.y + mapEnemies[i].object.height))
					{
						loseLife(fitzroy);			
					}
				}

				//check for enemy projectiles
				if (mapEnemies[i].projectile.exists)
				{
					if (fitzroy->object.position.x + fitzroy->object.width > mapEnemies[i].projectile.object.position.x &&
						fitzroy->object.position.x < mapEnemies[i].projectile.object.position.x + mapEnemies[i].projectile.object.width)
					{
						if (fitzroy->object.position.y + fitzroy->object.height > mapEnemies[i].projectile.object.position.y &&
							fitzroy->object.position.y < mapEnemies[i].projectile.object.position.y + mapEnemies[i].projectile.object.height)
						{
							if (mapEnemies[i].type == ENEMY_TYPE_DOWN_DROPPER)
							{
								loseLife(fitzroy);

								//todo work out what happens to the projectile after a collision
							}
							else
							{
								//laser will instakill
								fitzroy->lifeCount = 0;
							}
						}
					}
				}
			}
		}
	}

	//update the previous position value
	fitzroy->previousPosition.y = fitzroy->object.position.y;
	fitzroy->previousPosition.x = fitzroy->object.position.x;

	//calculate the x positions with an initial 0 velocity
	calculateOccupiedXGridPositions(fitzroy->object.position.x, 0, fitzroy->object.width);

	//caluclate the y positions based on the current y velocity
	calculateOccupiedYGridPositions(fitzroy->object.position.y, fitzroy->object.velocity.y, fitzroy->object.height);

	switch (fitzroy->state)
	{
		case JUMPING:
		{
			//apply some gravity on Y
			fitzroy->physicsYVelocity += GRAVITY;

			//adjust velocity if it is at reaches maximum
			adjustVelocityForLimit(fitzroy);

			//velocity is not 0 so we must be moving
			if (fitzroy->physicsYVelocity < 0)
			{
				//update physics position
				fitzroy->physicsYPosition += fitzroy->physicsYVelocity;

				//set new screen position
				temporaryPosition = fitzroy->physicsYPosition / 255;

				//calculate a new y position for jumping
				occupiedYTileTop_ = (temporaryPosition) / TILE_HEIGHT;
				
				//left and right are already calculated before the switch
				if (mapCells[MAPB(occupiedXTileLeft_, occupiedYTileTop_)].solid == FALSE && mapCells[MAPB(occupiedXTileRight_, occupiedYTileTop_)].solid == FALSE)
				{
					fitzroy->object.position.y = temporaryPosition;

					if(mapCells[MAPB(occupiedXTileLeft_, occupiedYTileTop_)].death || mapCells[MAPB(occupiedXTileRight_, occupiedYTileTop_)].death)
					{
						loseLife(fitzroy);
					}
				}
				else
				{
					if(hiddenFlag & REVEAL_TILE_TOUCH)
					{
						redrawCell(occupiedXTileLeft_, occupiedYTileTop_);
						redrawCell(occupiedXTileRight_, occupiedYTileTop_);
					}
				}
			}
			else
			{
				//reached the peak of the jump so we must be falling
				fitzroy->state = FALLING;
			}
		}
		break;

		case FALLING:
		{
			//apply some gravity on Y
			fitzroy->physicsYVelocity += FALL_GRAVITY;// GRAVITY;

			//adjust velocity to max out at terminal velocity
			adjustVelocityForLimit(fitzroy);

			//velocity is not 0 so we must be moving
			if (fitzroy->physicsYVelocity)
			{
				//update physics position
				fitzroy->physicsYPosition += fitzroy->physicsYVelocity;

				//set new screen position
				fitzroy->object.position.y = fitzroy->physicsYPosition / 255;

				//set new screen position
				temporaryPosition = fitzroy->physicsYPosition / 255;

				occupiedYTileBottom_ = (temporaryPosition + fitzroy->object.height) / TILE_HEIGHT;

				//left and right are already calculated before the switch									
				if (mapCells[MAPB(occupiedXTileLeft_, occupiedYTileBottom_)].solid == FALSE && mapCells[MAPB(occupiedXTileRight_, occupiedYTileBottom_)].solid == FALSE)
				{
					fitzroy->object.position.y = temporaryPosition;

					if(mapCells[MAPB(occupiedXTileLeft_, occupiedYTileBottom_)].death || mapCells[MAPB(occupiedXTileRight_, occupiedYTileBottom_)].death)
					{
						loseLife(fitzroy);
					}
				}
				else
				{
					if (hiddenFlag & REVEAL_TILE_TOUCH)
					{
						redrawCell(occupiedXTileLeft_, occupiedYTileBottom_);
						redrawCell(occupiedXTileRight_, occupiedYTileBottom_);
					}

					//once we colide with something we are now walking
					fitzroy->state = WALKING;

					//correct the vertical position so that the Y position is not through the tile
					fitzroy->object.position.y -= (fitzroy->object.position.y % TILE_HEIGHT);										
				}
			}
		}
		break;

		//falling off a tile
		case WALKING:
		{
			u8 belowOccupiedYTileBottom;

			//caluclate x position based on the change of x velocity
			calculateOccupiedXGridPositions(fitzroy->object.position.x, fitzroy->object.velocity.x, fitzroy->object.width);

			belowOccupiedYTileBottom = occupiedYTileBottom_ + 1;

			//check to see if the tile underneath fitzroy is not solid
			if (mapCells[MAPB(occupiedXTileLeft_, belowOccupiedYTileBottom)].solid == FALSE && mapCells[MAPB(occupiedXTileRight_, belowOccupiedYTileBottom)].solid == FALSE)
			{
				fitzroy->state = FALLING;

				if (mapCells[MAPB(occupiedXTileLeft_, belowOccupiedYTileBottom)].death || mapCells[MAPB(occupiedXTileRight_, belowOccupiedYTileBottom)].death)
				{
					loseLife(fitzroy);
				}
			}
			else if (hiddenFlag & REVEAL_TILE_TOUCH)
			{
				redrawCell(occupiedXTileLeft_, belowOccupiedYTileBottom);
				redrawCell(occupiedXTileRight_, belowOccupiedYTileBottom);
			}			
		}
		break;
	}
	
	//calculate grid y positions with a 0 change in velocity, which asumes the changes have already completed from earlier	
	if (fitzroy->state != FLYING)
	{
		calculateOccupiedYGridPositions(fitzroy->object.position.y, 0, fitzroy->object.height);
	}
	else
	{
		calculateOccupiedYGridPositions(fitzroy->object.position.y, fitzroy->object.velocity.y, fitzroy->object.height);		
	}	

	//calculate grid x position with the current x velocity
	calculateOccupiedXGridPositions(fitzroy->object.position.x, fitzroy->object.velocity.x, fitzroy->object.width);

	//new x position using x velocity
	if (fitzroy->object.velocity.x > 0) //right
	{
		if(fitzroy->direction != FITZROY_RIGHT)
		{
			fitzroy->changedDirection = TRUE;
			fitzroy->direction = FITZROY_RIGHT;
		}		

		if (fitzroy->object.position.x < SCREEN_X - fitzroy->object.width)
		{			
			if (mapCells[MAPB(occupiedXTileRight_, occupiedYTileTop_)].solid == FALSE && mapCells[MAPB(occupiedXTileRight_, occupiedYTileBottom_)].solid == FALSE)
			{
				fitzroy->object.position.x += fitzroy->object.velocity.x;

				if (mapCells[MAPB(occupiedXTileRight_, occupiedYTileTop_)].death && mapCells[MAPB(occupiedXTileRight_, occupiedYTileBottom_)].death)
				{
					loseLife(fitzroy);
				}
			}
			else
			{
				//fitzroy will attempt to use the hammer (if carried) on the wall if it is cracked
				if (hasItem(ITEM_TYPE_HAMMER))
				{
					useHammer(occupiedXTileRight_, occupiedYTileTop_);
					useHammer(occupiedXTileRight_, occupiedYTileBottom_);
				}
				
				if (hiddenFlag & REVEAL_TILE_TOUCH)
				{
					redrawCell(occupiedXTileRight_, occupiedYTileTop_);
					redrawCell(occupiedXTileRight_, occupiedYTileBottom_);
				}				
			}				
		}
	}
	else if (fitzroy->object.velocity.x < 0) //left
	{
		if(fitzroy->direction != FITZROY_LEFT)
		{
			fitzroy->changedDirection = TRUE;
			fitzroy->direction = FITZROY_LEFT;
		}

		if (fitzroy->object.position.x > 0)
		{
			if (mapCells[MAPB(occupiedXTileLeft_, occupiedYTileTop_)].solid == FALSE && mapCells[MAPB(occupiedXTileLeft_, occupiedYTileBottom_)].solid == FALSE)
			{
				fitzroy->object.position.x += fitzroy->object.velocity.x;

				if (mapCells[MAPB(occupiedXTileLeft_, occupiedYTileTop_)].death && mapCells[MAPB(occupiedXTileLeft_, occupiedYTileBottom_)].death)
				{
					loseLife(fitzroy);
				}
			}
			else
			{
				//fitzroy will attempt to use the hammer (if carried) on the wall if it is cracked
				if (hasItem(ITEM_TYPE_HAMMER))
				{
					useHammer(occupiedXTileLeft_, occupiedYTileTop_);
					useHammer(occupiedXTileLeft_, occupiedYTileBottom_);
				}
				
				if (hiddenFlag & REVEAL_TILE_TOUCH)
				{
					redrawCell(occupiedXTileLeft_, occupiedYTileTop_);
					redrawCell(occupiedXTileLeft_, occupiedYTileBottom_);
				}				
			}
		}
	}

	if (fitzroy->state == FLYING)
	{
		if (fitzroy->object.velocity.y > 0)
		{
			if (mapCells[MAPB(occupiedXTileLeft_, occupiedYTileBottom_)].solid == FALSE && mapCells[MAPB(occupiedXTileRight_, occupiedYTileBottom_)].solid == FALSE)
			{
				fitzroy->object.position.y += fitzroy->object.velocity.y;

				if (mapCells[MAPB(occupiedXTileLeft_, occupiedYTileBottom_)].death && mapCells[MAPB(occupiedXTileRight_, occupiedYTileBottom_)].death)
				{
					loseLife(fitzroy);
				}
			}
			else
			{
				//fitzroy will attempt to use the hammer (if carried) on the wall if it is cracked
				if (hasItem(ITEM_TYPE_HAMMER))
				{
					useHammer(occupiedXTileLeft_, occupiedYTileBottom_);
					useHammer(occupiedXTileRight_, occupiedYTileBottom_);
				}
				
				if (hiddenFlag & REVEAL_TILE_TOUCH)
				{
					redrawCell(occupiedXTileLeft_, occupiedYTileBottom_);
					redrawCell(occupiedXTileRight_, occupiedYTileBottom_);
				}				
			}
		}
		else if (fitzroy->object.velocity.y < 0)
		{
			if (mapCells[MAPB(occupiedXTileLeft_, occupiedYTileTop_)].solid == FALSE && mapCells[MAPB(occupiedXTileRight_, occupiedYTileTop_)].solid == FALSE)
			{
				fitzroy->object.position.y += fitzroy->object.velocity.y;

				if (mapCells[MAPB(occupiedXTileLeft_, occupiedYTileTop_)].death && mapCells[MAPB(occupiedXTileRight_, occupiedYTileTop_)].death)
				{
					loseLife(fitzroy);
				}
			}
			else
			{
				//fitzroy will attempt to use the hammer (if carried) on the wall if it is cracked
				if (hasItem(ITEM_TYPE_HAMMER))
				{
					useHammer(occupiedXTileLeft_, occupiedYTileTop_);
					useHammer(occupiedXTileRight_, occupiedYTileTop_);
				}
				
				if (hiddenFlag & REVEAL_TILE_TOUCH)
				{
					redrawCell(occupiedXTileLeft_, occupiedYTileTop_);
					redrawCell(occupiedXTileRight_, occupiedYTileTop_);
				}
			}
		}
	}

	processCell(fitzroy, keys, occupiedXTileLeft_, occupiedYTileTop_);
	processCell(fitzroy, keys, occupiedXTileLeft_, occupiedYTileBottom_);
	processCell(fitzroy, keys, occupiedXTileRight_, occupiedYTileTop_);
	processCell(fitzroy, keys, occupiedXTileRight_, occupiedYTileBottom_);
	
	//velocity set to 0 after position changed
	fitzroy->object.velocity.x = 0;
	fitzroy->object.velocity.y = 0;

	//more testing for jumping
	fitzroy->physicsXPosition = fitzroy->object.position.x * 255;
	fitzroy->physicsYPosition = fitzroy->object.position.y * 255;

	//check lifeCount
    if (fitzroy->lifeCount <= 0)
    {
        //lifecount 0 so game over
        gameRunning = FALSE;
        gameResult = LOSS;
    }
}

void adjustVelocityForLimit(Fitzroy* fitzroy)
{
	//going down
	if (fitzroy->physicsYVelocity > 0)
	{
		if (fitzroy->physicsYVelocity > TERMINAL_VELOCITY)
		{
			//terminal velocity
			fitzroy->physicsYVelocity = TERMINAL_VELOCITY;
		}
		else if (fitzroy->physicsYVelocity < 32)
		{
			fitzroy->physicsYVelocity = 0;
		}
	}
	else if (fitzroy->physicsYVelocity < 0) //going up
	{
		if (fitzroy->physicsYVelocity < MAX_VERTICAL_VELOCITY)
		{
			//max vertical velocity
			fitzroy->physicsYVelocity = MAX_VERTICAL_VELOCITY;
		}
		else if (fitzroy->physicsYVelocity > -32)
		{
			fitzroy->physicsYVelocity = 0;
		}
	}
}

void moveFitzroy(Fitzroy* fitzroy, Keys* keys)
{
	if (keys->upIsDown)
	{
		fitzroy->object.velocity.y = -1;
	}

	if (keys->downIsDown)
	{
		fitzroy->object.velocity.y = 1;
	}

	if (keys->leftIsDown)
	{
		fitzroy->object.velocity.x = -1;
	}

	if (keys->rightIsDown)
	{
		fitzroy->object.velocity.x = 1;
	}

	if (keys->jumpIsDown)
	{
		//only jump from walking state as we don't want multijump
		if (fitzroy->state == WALKING)
		{
			fitzroy->state = JUMPING;

			if(hasItem(ITEM_TYPE_SHOES))
			{
				fitzroy->physicsYVelocity = SHOES_JUMP_VELOCITY;
			}
			else
			{
				//set initial jump velocity
				fitzroy->physicsYVelocity = INITIAL_JUMP_VELOCITY;
			}			
		}
	}
}

void processCell(Fitzroy* fitzroy, Keys* keys, u8 x, u8 y)
{
	Cell* cell = &mapCells[MAPB(x, y)];
	
	if(cell->graphicType == GRAPHIC_TYPE_ITEM)
	{
		//item detected so now it can be picked up and put into the inventory
		pickupItem(&mapItems[cell->index]);
	
		//set the graphic to empty so it is no longer displayed on redraw
		cell->graphicType = GRAPHIC_TYPE_EMPTY;
	
		//clear the content of the cell
		clearCell(x, y);

		//if fitzroy has the chair he now switches to flying mode
		if(cell->graphicNumber.item == ITEM_TYPE_CHAIR)
		{
			fitzroy->state = FLYING;
		}
	}
	else if(cell->graphicType == GRAPHIC_TYPE_INTERACTABLE)
	{
		processInteractable(fitzroy, keys, cell->index, x, y);

		if(fitzroy->previousPosition.y != fitzroy->object.position.y || fitzroy->previousPosition.x != fitzroy->object.position.x)
		{
			//redraw the cell if fitzroy moves over it
			redrawCellOfType(x, y, cell->graphicType);
		}
	}
}

void processInteractable(Fitzroy* fitzroy, Keys* keys, u8 index, u8 x, u8 y)
{
	//check to see if use key is activated
	if(keys->useIsDown)
	{
		//call the activate function on the interactable
		activateInteractable(&mapInteractables[index], ACTIVATION_SOURCE_FITZROY);
	}

	//door follows a different rule in terms of touching it
	if (mapInteractables[index].type == INTERACTABLE_TYPE_DOOR || mapInteractables[index].type == INTERACTABLE_TYPE_TRAP_DOOR)
	{
		//when fitzroy is carrying a key we change the door state to off (i.e open)
		if (hasItem(ITEM_TYPE_KEY))
		{
			//set the cell to empty as the door is now open
			mapCells[MAPB(x, y)].graphicType = GRAPHIC_TYPE_EMPTY;

			//clear the door square
			clearCell(x, y);
		}
		else
		{
			//reset fitzroy x position to the previous position as the door is impassable (left or right doors)
			if (mapInteractables[index].type == INTERACTABLE_TYPE_DOOR)
			{
				fitzroy->object.position.x = fitzroy->previousPosition.x;
			}
			else
			{
				//reset fitzroy state to walking as he is no longer falling
				fitzroy->state = WALKING;							

				//reset fitzroy y position to the previous position as the door is impassable (going through trapdoor from above)
				fitzroy->object.position.y = fitzroy->previousPosition.y;	
			}
		}
	}
	else if(mapInteractables[index].type == INTERACTABLE_TYPE_DOOR2 || mapInteractables[index].type == INTERACTABLE_TYPE_TRAP_DOOR2)
	{
		//when fitzroy is carrying a key2 we change the door graphic to empty
		if (hasItem(ITEM_TYPE_KEY2))
		{
			//set the cell to empty as the door is now open
			mapCells[MAPB(x, y)].graphicType = GRAPHIC_TYPE_EMPTY;

			//clear the door square
			clearCell(x, y);
		}
		else
		{
			//reset fitzroy x position to the previous position as the door is impassable (left or right doors)
			if (mapInteractables[index].type == INTERACTABLE_TYPE_DOOR2)
			{
				fitzroy->object.position.x = fitzroy->previousPosition.x;
			}
			else
			{
				//reset fitzroy state to walking as he is no longer falling
				fitzroy->state = WALKING;							

				//reset fitzroy y position to the previous position as the door is impassable (going through trapdoor from above)
				fitzroy->object.position.y = fitzroy->previousPosition.y;	
			}
		}
	}
	else if(mapInteractables[index].type == INTERACTABLE_TYPE_DOOR_ACTIVATABLE)
	{
		//activatable doors are solid while they are active
		if (mapInteractables[index].state == INTERACTABLE_STATE_ON)
		{
			fitzroy->object.position.x = fitzroy->previousPosition.x;
		}
	}
}

//todo change to bitshift >> 2to improve performance
void calculateOccupiedXGridPositions(u8 xPosition, i8 xVelocity, u8 width)
{	
	occupiedXTileLeft_ = (xPosition + xVelocity) / TILE_WIDTH_BYTES;

	//the -1 is because when you add the width we end up going over which gives us a different tile to what we want
	occupiedXTileRight_ = (xPosition + width + xVelocity - 1) / TILE_WIDTH_BYTES;
}

//todo change to bitshift >> 2to improve performance
void calculateOccupiedYGridPositions(u8 yPosition, i8 yVelocity, u8 height)
{
	occupiedYTileTop_ = (yPosition + yVelocity) / TILE_HEIGHT;

	//the -1 is because when you add the height we end up going over which gives us a different tile to what we want
	occupiedYTileBottom_ = (yPosition + height + yVelocity - 1) / TILE_HEIGHT;
}

void redrawPreviouslyOccupiedCells()
{
	redrawCell(occupiedXTileLeft_, occupiedYTileBottom_);
	redrawCell(occupiedXTileRight_, occupiedYTileBottom_);
	redrawCell(occupiedXTileLeft_, occupiedYTileTop_);
	redrawCell(occupiedXTileRight_, occupiedYTileTop_);
}

void useHammer(u8 x, u8 y)
{
	if (mapCells[MAPB(x, y)].broken)
	{
		mapCells[MAPB(x, y)].graphicType = GRAPHIC_TYPE_EMPTY;
		mapCells[MAPB(x, y)].solid = FALSE;
		clearCell(x, y);
	}
}

void loseLife(Fitzroy* fitzroy)
{
	//only lose a life if the invincibility time is not active
	if(invincibilityActive == FALSE)
	{
		//enemy projectile detected so lose a life
		--fitzroy->lifeCount;

		//force a redraw of the hud because of the change in lifecount
		gameHud.redraw = TRUE;

		//fitzroy becomes invincibile when he loses a life so he doesn't lose all the lives at once
		startInvincibilityTimer();
	}
}