#ifndef FITZROY_H
#define FITZROY_H

#include <cpctelera.h>
#include "Object.h"
#include "Keys.h"
#include "Map.h"
#include "Timer.h"
#include "Cell.h"
#include "sprites/FitzroySprites.h"
#include "Mediator.h"
#include "Hidden.h"

typedef struct Fitzroy Fitzroy;

enum State {WALKING, JUMPING, FALLING, FLYING};

typedef enum State State;

struct Fitzroy
{
    Object object;
    Position previousPosition;
    i16 physicsXVelocity;
    i16 physicsYVelocity;
    u16 physicsXPosition;
    u16 physicsYPosition;
    State state;
    i8 lifeCount;
    u8 changedDirection : 1;
    i8 direction : 2;
};

extern Fitzroy fitzroy;

void initFitzroy(Fitzroy* fitzroy);
void drawFitzroy(Fitzroy* fitzroy);
void logicFitzroy(Fitzroy* fitzroy, Keys* keys);
void moveFitzroy(Fitzroy* fitzroy, Keys* keys);

void adjustVelocityForLimit(Fitzroy* fitzroy);

void processCell(Fitzroy* fitzroy, Keys* keys, u8 x, u8 y);
void processInteractable(Fitzroy* fitzroy, Keys* keys, u8 index, u8 x, u8 y);

void calculateOccupiedXGridPositions(u8 xPosition, i8 xVelocity, u8 width);
void calculateOccupiedYGridPositions(u8 yPosition, i8 yVelocity, u8 height);

//redraw the cells that have been covered up by removing Fitzroy's tail
void redrawPreviouslyOccupiedCells();

//use the hammer (if it is being carried) on a particular cell in the grid
void useHammer(u8 x, u8 y);

//deduct a life from fitzroy and refresh the hud (activates the invincibility timer)
void loseLife(Fitzroy* fitzroy);

#endif