#include "Game.h"

Keys keys_;

#define EXIT_DIALOG_WIDTH_ 38
#define EXIT_DIALOG_HEIGHT_ 20
#define EXIT_DIALOG_X_ 22
const u8 exitText_[10] = {"Exit? Y/N\0"};

void initGame(u8 selectedLevel)
{
    cpct_setInterruptHandler(interruptHandler);    
    initKeys(&keys_);
    initMap(selectedLevel);
    initFitzroy(&fitzroy);

	//setup hud
	initHud(&gameHud);

    //set the palette to earty here for now
    cpct_setPalette(EARTHY_PALETTE, PALETTE_COLOUR_COUNT);

    gameRunning = TRUE;
}

void startGame()
{
	gameResult = ACTIVE;

    //set background and border colour
    cpct_clearScreen(backgroundColour);
    cpct_setBorder(0x14);

    //due to a lack of tuning the timer is now disabled
    //only start the timer if we are on max difficulty
    //if(difficultyLevel == MAX_DIFFICULTY)
    //{
    //    startLevelTimer();
    //}	

    //game loop
    while (gameRunning)
    {
        //read the input from the player
        readInput();

        //game logic
        logicGame();

        //wait for vsync
        cpct_waitVSYNC();

        //draw game
        drawGame();
    }
}

void logicGame()
{
    //move fitzroy position
    moveFitzroy(&fitzroy, &keys_);

    //fitzroy logic
    logicFitzroy(&fitzroy, &keys_);

    //map logic
    logicMap();
}

void readInput()
{
    if(cpct_isAnyKeyPressed_f)
    {
        //up
        if(cpct_isKeyPressed(controlsArray[CONTROLS_UP]))
        {
            keys_.upIsDown = TRUE;
        }
        else
        {
            keys_.upIsDown = FALSE;
        }

        //down
        if(cpct_isKeyPressed(controlsArray[CONTROLS_DOWN]))
        {
            keys_.downIsDown = TRUE;
        }
        else
        {
            keys_.downIsDown = FALSE;
        }

        //left
        if(cpct_isKeyPressed(controlsArray[CONTROLS_LEFT]))
        {
            keys_.leftIsDown = TRUE;
        }
        else
        {
            keys_.leftIsDown = FALSE;
        }

        //right
        if(cpct_isKeyPressed(controlsArray[CONTROLS_RIGHT]))
        {
            keys_.rightIsDown = TRUE;
        }
        else
        {
            keys_.rightIsDown = FALSE;
        }

        //jump
        if(cpct_isKeyPressed(controlsArray[CONTROLS_JUMP]))
        {
            keys_.jumpIsDown = TRUE;
        }
        else
        {
            keys_.jumpIsDown = FALSE;
        }

        //use
        if(cpct_isKeyPressed(controlsArray[CONTROLS_USE]))
        {
            keys_.useIsDown = TRUE;
        }
        else
        {
            keys_.useIsDown = FALSE;
        }

        if(cpct_isKeyPressed(Key_Esc))
        {
            if(confirmQuit())
            {
                gameRunning = FALSE;
                gameResult = QUIT; 
            }
            else
            {
                gameRunning = TRUE;
            }
        }
    }    
}

//this is called each time an interupt occurs
void interruptHandler()
{
    //check for user input
    cpct_scanKeyboard_if();

    //timer
    timerInterruptHandler();
}

void drawGame()
{
    drawFitzroy(&fitzroy);

	drawMap();

    //hud will only update if redraw is true
    gameHud.drawHud(&gameHud);
}

//confirm with the player if they want to quit the game
u8 confirmQuit()
{    
    u8* memory = cpct_getScreenPtr(CPCT_VMEM_START, EXIT_DIALOG_X_, 40);
    cpct_drawSolidBox(memory, cpct_px2byteM0(0, 0) , EXIT_DIALOG_WIDTH_, EXIT_DIALOG_HEIGHT_ );
    
    memory = cpct_getScreenPtr(CPCT_VMEM_START, EXIT_DIALOG_X_ + 1, 45);
    cpct_drawStringM0(exitText_, memory, 1, 0);

    //break out of the loop once the user decides
    while(1)
    {
        cpct_scanKeyboard();

        if(cpct_isKeyPressed(Key_Y))
        {
            return TRUE;  
        }
        else if(cpct_isKeyPressed(Key_N))
        {            
            //clear just the dialog
            memory = cpct_getScreenPtr(CPCT_VMEM_START, EXIT_DIALOG_X_, 40);
            cpct_drawSolidBox(memory, cpct_px2byteM0(5, 5) , EXIT_DIALOG_WIDTH_, EXIT_DIALOG_HEIGHT_ );

            //force a redraw of the map
            mapRedraw = TRUE;
            return FALSE;
        }
    }
}