#include "Hud.h"

#define HUD_X_POSITION_ 0
#define HUD_X_POSITION_TWO_ 64
#define HUD_Y_POSITION_ 192
#define HUD_WIDTH_ONE_ 64
#define HUD_WIDTH_TWO_ 16
#define HUD_HEIGHT_ 8

#define HUD_SCORE_START_ 0
#define LIFECOUNT_START_ 21
#define HUD_SPACE_ 1

#define ASCII_TO_NUMBER_OFFSET_ 48

#define HUD_KEY_START_ 30
#define HUD_SPANNER_START_ 34
#define HUD_KEYCARD_START_ 38
#define HUD_HAMMER_START_ 42
#define HUD_SHOES_START_ 47
#define HUD_MAGNIFYING_GLASS_START_ 51
#define HUD_KEY2_START_ 55
#define HUD_CHAIR_START_ 59
#define HUD_TIMER_CLOCK_START_ 64
#define HUD_TIMER_TEXT_START_ 68
#define HUD_ITEM_WIDTH_BYTES_ 4

#define KEY_FLAG_ 1
#define SPANNER_FLAG_ 2
#define KEYCARD_FLAG_ 4
#define HAMMER_FLAG_ 8
#define SHOES_FLAG_ 16
#define MAGNIFYING_GLASS_FLAG_ 32
#define KEY2_FLAG_ 64
#define CHAIR_FLAG_ 128

void _drawHud(Hud* hud);

Hud gameHud;
u8 scoreHud[6];
u8 lifeCountHud_[2];
u8 timerHud_[4];

u8 displayedItems_ = 0;

void initHud(Hud* hud)
{
    hud->drawHud = _drawHud;
    hud->redraw = TRUE;    

    //setup the lifecount hud
    lifeCountHud_[0] = fitzroy.lifeCount + ASCII_TO_NUMBER_OFFSET_;
    lifeCountHud_[1] = '\0'; 

    timerHud_[0] = 0 + ASCII_TO_NUMBER_OFFSET_;
    timerHud_[1] = 0 + ASCII_TO_NUMBER_OFFSET_;
    timerHud_[2] = 0 + ASCII_TO_NUMBER_OFFSET_;
    timerHud_[3] = '\0';

    //items don't carry across levels
    displayedItems_ = 0;
}

void _drawHud(Hud* hud)
{    
	if (hud->redraw)
	{
		u8* hudVideoMemory;

		//draw the hud strip on the background in two parts
		hudVideoMemory = cpct_getScreenPtr(CPCT_VMEM_START, HUD_X_POSITION_, HUD_Y_POSITION_);
		cpct_drawSolidBox(hudVideoMemory, backgroundColour, HUD_WIDTH_ONE_, HUD_HEIGHT_);

		hudVideoMemory = cpct_getScreenPtr(CPCT_VMEM_START, HUD_X_POSITION_TWO_, HUD_Y_POSITION_);
		cpct_drawSolidBox(hudVideoMemory, backgroundColour, HUD_WIDTH_TWO_, HUD_HEIGHT_);

		//highscore
		convertIntToChar(hud->gameScore, scoreHud);
		hudVideoMemory = cpct_getScreenPtr(CPCT_VMEM_START, HUD_SCORE_START_, HUD_Y_POSITION_);
		cpct_drawStringM0(scoreHud, hudVideoMemory, 1, 0);

        //life count hud        
        hudVideoMemory = cpct_getScreenPtr(CPCT_VMEM_START, LIFECOUNT_START_, HUD_Y_POSITION_);
        cpct_drawSprite(G_hud_fitzroy, hudVideoMemory, HUD_ITEM_WIDTH_BYTES_, HUD_HEIGHT_);

        hudVideoMemory = cpct_getScreenPtr(CPCT_VMEM_START, LIFECOUNT_START_ + HUD_ITEM_WIDTH_BYTES_, HUD_Y_POSITION_);
        lifeCountHud_[0] = fitzroy.lifeCount + ASCII_TO_NUMBER_OFFSET_;
        cpct_drawStringM0(lifeCountHud_, hudVideoMemory, 1, 0);

        //red key
        if(displayedItems_ & KEY_FLAG_)
        {
            hudVideoMemory = cpct_getScreenPtr(CPCT_VMEM_START, HUD_KEY_START_, HUD_Y_POSITION_);
            cpct_drawSprite(G_hud_key, hudVideoMemory, HUD_ITEM_WIDTH_BYTES_, HUD_HEIGHT_);
        }

        //spanner
        if(displayedItems_ & SPANNER_FLAG_)
        {
            hudVideoMemory = cpct_getScreenPtr(CPCT_VMEM_START, HUD_SPANNER_START_, HUD_Y_POSITION_);
            cpct_drawSprite(G_hud_spanner, hudVideoMemory, HUD_ITEM_WIDTH_BYTES_, HUD_HEIGHT_);
        }

        //key card
        if(displayedItems_ & KEYCARD_FLAG_)
        {
            hudVideoMemory = cpct_getScreenPtr(CPCT_VMEM_START, HUD_KEYCARD_START_, HUD_Y_POSITION_);
            cpct_drawSprite(G_hud_keycard, hudVideoMemory, HUD_ITEM_WIDTH_BYTES_, HUD_HEIGHT_);
        }

        //hammer
        if(displayedItems_ & HAMMER_FLAG_)
        {
            hudVideoMemory = cpct_getScreenPtr(CPCT_VMEM_START, HUD_HAMMER_START_, HUD_Y_POSITION_);
            cpct_drawSprite(G_hud_hammer, hudVideoMemory, HUD_ITEM_WIDTH_BYTES_, HUD_HEIGHT_);
        }

        //shoes
        if(displayedItems_ & SHOES_FLAG_)
        {
            hudVideoMemory = cpct_getScreenPtr(CPCT_VMEM_START, HUD_SHOES_START_, HUD_Y_POSITION_);
            cpct_drawSprite(G_hud_shoes, hudVideoMemory, HUD_ITEM_WIDTH_BYTES_, HUD_HEIGHT_);
        }

        //magnifying glass
        if(displayedItems_ & MAGNIFYING_GLASS_FLAG_)
        {
            hudVideoMemory = cpct_getScreenPtr(CPCT_VMEM_START, HUD_MAGNIFYING_GLASS_START_, HUD_Y_POSITION_);
            cpct_drawSprite(G_hud_magnifying_glass, hudVideoMemory, HUD_ITEM_WIDTH_BYTES_, HUD_HEIGHT_);
        }

        //key2 (blueish key)
        if(displayedItems_ & KEY2_FLAG_)
        {
            hudVideoMemory = cpct_getScreenPtr(CPCT_VMEM_START, HUD_KEY2_START_, HUD_Y_POSITION_);
            cpct_drawSprite(G_hud_key2, hudVideoMemory, HUD_ITEM_WIDTH_BYTES_, HUD_HEIGHT_);
        }

        //chair
        if(displayedItems_ & CHAIR_FLAG_)
        {
            hudVideoMemory = cpct_getScreenPtr(CPCT_VMEM_START, HUD_CHAIR_START_, HUD_Y_POSITION_);
            cpct_drawSprite(G_hud_chair, hudVideoMemory, HUD_ITEM_WIDTH_BYTES_, HUD_HEIGHT_);
        }

        //timer remaining        
        drawGameTimer(hud);
        
		hud->redraw = FALSE;
	}    
}

void addToScore(Hud* hud, u16 addedScore)
{
    hud->gameScore += addedScore;
    hud->redraw = TRUE;
}

//converts an input u16 to an array of u8 which is assumed to have 6 elements, 5 digits and \0
void convertIntToChar(u16 input, u8 output[6])
{    
    //need to add 48 to each number to actually display the correct number on screen    
    u8 toCharPosition = 0;
    u8 nextDigit = FALSE;

    if(input >= 10000)
    {                        
        output[toCharPosition] = (input / 10000) + ASCII_TO_NUMBER_OFFSET_;
        ++toCharPosition;
        input = (input % 10000);
        nextDigit = TRUE;
    }

    if(input >= 1000 || nextDigit)
    {                        
        output[toCharPosition] = (input / 1000) + ASCII_TO_NUMBER_OFFSET_;
        ++toCharPosition;
        input = (input % 1000);
        nextDigit = TRUE;
    }

    if(input >= 100 || nextDigit)
    {                             
        output[toCharPosition] = (input / 100) + ASCII_TO_NUMBER_OFFSET_;
        ++toCharPosition;
        input = (input % 100);
        nextDigit = TRUE;        
    }

    if(input >= 10 || nextDigit)
    {
        output[toCharPosition] = (input / 10) + ASCII_TO_NUMBER_OFFSET_;
        ++toCharPosition;
        input = (input % 10);
        nextDigit = TRUE;
    }
    
    if(input < 10 || nextDigit)
    {        
        output[toCharPosition] = input + ASCII_TO_NUMBER_OFFSET_;
        ++toCharPosition;
    }

    //add the null terminator
    output[toCharPosition] = '\0';
}

void addItemToHud(Hud* hud, u8 itemType)
{
    switch(itemType)
    {
        case ITEM_TYPE_KEY:
        {
            displayedItems_ |= KEY_FLAG_;
        }
        break;

        case ITEM_TYPE_SPANNER:
        {
            displayedItems_ |= SPANNER_FLAG_;
        }
        break;

        case ITEM_TYPE_KEYCARD:
        {
            displayedItems_ |= KEYCARD_FLAG_;
        }
        break;

        case ITEM_TYPE_HAMMER:
        {
            displayedItems_ |= HAMMER_FLAG_;
        }
        break;

        case ITEM_TYPE_SHOES:
        {
            displayedItems_ |= SHOES_FLAG_;
        }
        break;

        case ITEM_TYPE_MAGNIFYING_GLASS:
        {
            displayedItems_ |= MAGNIFYING_GLASS_FLAG_;
        }
        break;

        case ITEM_TYPE_KEY2:
        {
            displayedItems_ |= KEY2_FLAG_;
        }
        break;

        case ITEM_TYPE_CHAIR:
        {
            displayedItems_ |= CHAIR_FLAG_;
        }
        break;
    }
}

void removeItemFromHud(Hud* hud, u8 itemType)
{
    switch(itemType)
    {
        case ITEM_TYPE_KEY:
        {
            displayedItems_ &= ~KEY_FLAG_;
        }
        break;
        
        case ITEM_TYPE_KEY2:
        {
            displayedItems_ &= ~KEY2_FLAG_;
        }
        break;

        case ITEM_TYPE_KEYCARD:
        {
            displayedItems_ &= ~KEYCARD_FLAG_;
        }
        break;

        case ITEM_TYPE_SPANNER:
        {
            displayedItems_ &= ~SPANNER_FLAG_;
        }
        break;
    }
}

void drawGameTimer(Hud* hud)
{
    //only draw the timer if we are actually using it
    if(levelTimerActive)
    {
        u8* hudVideoMemory;
        
        hudVideoMemory = cpct_getScreenPtr(CPCT_VMEM_START, HUD_TIMER_CLOCK_START_, HUD_Y_POSITION_);
        cpct_drawSprite(G_hud_timer, hudVideoMemory, HUD_ITEM_WIDTH_BYTES_, HUD_HEIGHT_);
    
        hudVideoMemory = cpct_getScreenPtr(CPCT_VMEM_START, HUD_TIMER_TEXT_START_, HUD_Y_POSITION_);
        timerIntToChar(levelTimerSeconds, timerHud_);
        cpct_drawStringM0(timerHud_, hudVideoMemory, 1, 0);
    }    
}

void timerIntToChar(u8 input, u8 output[4])
{
    if(input < 10)
    {
        output[0] = 0 + ASCII_TO_NUMBER_OFFSET_;
        output[1] = 0 + ASCII_TO_NUMBER_OFFSET_;
        output[2] = input + ASCII_TO_NUMBER_OFFSET_;        
    }

    if(input < 100)
    {
        output[0] = 0 + ASCII_TO_NUMBER_OFFSET_;
        output[1] = (input / 10) + ASCII_TO_NUMBER_OFFSET_;
        output[2] = (input % 10) + ASCII_TO_NUMBER_OFFSET_;        
    }

    if(input >= 100)
    {
        output[0] = (input / 100) + ASCII_TO_NUMBER_OFFSET_;
        input = (input % 100);
        output[1] = (input / 10) + ASCII_TO_NUMBER_OFFSET_;
        output[2] = (input % 10) + ASCII_TO_NUMBER_OFFSET_;
    }

    output[3] = '\0';
}