#include "Interactable.h"

#define MAPB(x,y) x*SCREEN_TILES_Y + y

void drawInteractable(Interactable* interactable)
{
    if(interactable->visible)
    {
        //draw the specified interactable in the interactable graphic set
        u8* interactableVideoMemory = cpct_getScreenPtr(CPCT_VMEM_START, interactable->position.x * TILE_WIDTH_BYTES, interactable->position.y * TILE_HEIGHT);
    
        //todo handle types and graphics better for drawing interactables
        cpct_drawSprite(interactableGraphicSet[interactable->type], interactableVideoMemory, TILE_WIDTH_BYTES, TILE_HEIGHT);
	}
	
	//draw an x if the interactable is in a broken state
	if(interactable->broken)
	{
		u8* interactableVideoMemory = cpct_getScreenPtr(CPCT_VMEM_START, interactable->position.x * TILE_WIDTH_BYTES, interactable->position.y * TILE_HEIGHT);

		cpct_drawSpriteMasked (G_broken_interactable_indicator, interactableVideoMemory, TILE_WIDTH_BYTES, TILE_HEIGHT);		
	}
}

void activateInteractable(Interactable* interactable, u8 activationSource)
{
	//set interactable to unbroken if carrying a spanner
	if(interactable->broken)
	{
		if(hasItem(ITEM_TYPE_SPANNER))
		{
			interactable->broken = FALSE;
			mapCells[MAPB(interactable->position.x, interactable->position.y)].broken = FALSE;
		}
	}

	//make sure that the interactable has not yet been used and it is currently not broken
	if (interactable->used == FALSE && interactable->broken == FALSE)
	{
		//set interactable as used so it can only be activated once
		interactable->used = TRUE;

		//update the enemy state if the type is a switch (putting both active and inactive here for now)
		if (interactable->type == INTERACTABLE_TYPE_SWITCH || interactable->type == INTERACTABLE_TYPE_SWITCH_ACTIVE)
		{
			//toggle between on and off
			if (interactable->state == INTERACTABLE_STATE_OFF)
			{
				interactable->state = INTERACTABLE_STATE_ON;

				//change the type to do a graphic change
				interactable->type = INTERACTABLE_TYPE_SWITCH_ACTIVE;
				
				//change the graphic to the on switch
				mapCells[MAPB(interactable->position.x, interactable->position.y)].graphicNumber.interactable = INTERACTABLE_TYPE_SWITCH_ACTIVE;

				//redraw the cell after a graphic change
				redrawCell(interactable->position.x, interactable->position.y);
			}
			else
			{
				interactable->state = INTERACTABLE_STATE_OFF;

				//change the type to do a graphic change
				interactable->type = INTERACTABLE_TYPE_SWITCH;

				//change the graphic to the off switch
				mapCells[MAPB(interactable->position.x, interactable->position.y)].graphicNumber.interactable = INTERACTABLE_TYPE_SWITCH;

				//redraw the cell after a graphic change
				redrawCell(interactable->position.x, interactable->position.y);
			}

			//make sure a valid enemy link is in place
			if (interactable->enemyLink != UNDEFINED_LINK)
			{
				//change the state of the enemy
				activateEnemy(&mapEnemies[interactable->enemyLink]);
			}

			//make sure a valid interactable link is in place
			//in theory we could have both an enemy link and an interactable link but this is not intended
			if (interactable->interactableLink != UNDEFINED_LINK)
			{
				//change the state of a connected interactable
				activateInteractable(&mapInteractables[interactable->interactableLink], ACTIVATION_SOURCE_LINK);
			}
		}
		else if (interactable->type == INTERACTABLE_TYPE_DOOR || interactable->type == INTERACTABLE_TYPE_TRAP_DOOR)
		{
			//activate interactable should only be called for a door when Fitzroy is carrying a key
			if (hasItem(ITEM_TYPE_KEY))
			{
				//might not need to use state for doors, visibility may be better
				interactable->state = INTERACTABLE_STATE_OFF;
				interactable->visible = FALSE;

				//clear the interactable from screen as it is no longer visibile
				clearInteractable(interactable);
			}
		}
		else if (interactable->type == INTERACTABLE_TYPE_DOOR_ACTIVATABLE)
		{
			//an activatable door can only be activated from a linked interactable
			if (activationSource == ACTIVATION_SOURCE_LINK)
			{				
				//might not need to use state for doors, visibility may be better
				interactable->state = (interactable->state == INTERACTABLE_STATE_OFF ? INTERACTABLE_STATE_ON : INTERACTABLE_STATE_OFF);
				interactable->visible = !interactable->visible;

				//clear the interactable from screen as it is no longer visibile
				clearInteractable(interactable);

				//set graphic type to empty
				mapCells[MAPB(interactable->position.x, interactable->position.y)].graphicType = GRAPHIC_TYPE_EMPTY;

				//clear the cell
				clearCell(interactable->position.x, interactable->position.y);
			}
			else
			{
				interactable->used = FALSE;
			}
		}
		else if (interactable->type == INTERACTABLE_TYPE_CARD_READER || interactable->type == INTERACTABLE_TYPE_CARD_READER_ACTIVE)
		{
			//when fitzroy is carrying a keycard we need to activate the linked interactable
			if (hasItem(ITEM_TYPE_KEYCARD))
			{
				//make sure a valid enemy link is in place
				if (interactable->enemyLink != UNDEFINED_LINK)
				{
					//change the state of the enemy
					activateEnemy(&mapEnemies[interactable->enemyLink]);
				}

				//make sure a valid interactable link is in place
				//in theory we could have both an enemy link and an interactable link but this is not intended
				if (interactable->interactableLink != UNDEFINED_LINK)
				{
					//change the state of a connected interactable
					activateInteractable(&mapInteractables[interactable->interactableLink], ACTIVATION_SOURCE_LINK);
				}

				//toggle between on and off
				if (interactable->state == INTERACTABLE_STATE_OFF)
				{
					interactable->state = INTERACTABLE_STATE_ON;

					//change the type to do a graphic change
					interactable->type = INTERACTABLE_TYPE_CARD_READER_ACTIVE;

					//change the graphic to the on card reader
					mapCells[MAPB(interactable->position.x, interactable->position.y)].graphicNumber.interactable = INTERACTABLE_TYPE_CARD_READER_ACTIVE;

					//redraw the cell after a graphic change
					redrawCell(interactable->position.x, interactable->position.y);
				}
				else
				{
					interactable->state = INTERACTABLE_STATE_OFF;

					//change the type to do a graphic change
					interactable->type = INTERACTABLE_TYPE_CARD_READER;

					//change the graphic to the off card reader
					mapCells[MAPB(interactable->position.x, interactable->position.y)].graphicNumber.interactable = INTERACTABLE_TYPE_CARD_READER;

					//redraw the cell after a graphic change
					redrawCell(interactable->position.x, interactable->position.y);
				}
			}
		}
		else if (interactable->type == INTERACTABLE_TYPE_EXIT)
		{
			//set result to win as the level is complete
			gameResult = WIN;

			//stop the game running
			gameRunning = FALSE;
		}
	}
}

void clearInteractable(Interactable* interactable)
{
    //draw over where the interactable was on the screen
	u8* memory = cpct_getScreenPtr(CPCT_VMEM_START, interactable->position.x * TILE_WIDTH_BYTES, interactable->position.y * TILE_HEIGHT);
    
    cpct_drawSolidBox(memory, cpct_px2byteM0(2, 2), TILE_WIDTH_BYTES, TILE_HEIGHT);
}