#ifndef INTERACTABLE_H
#define INTERACTABLE_H

#include <cpctelera.h>
#include "Object.h"
#include "GameTypes.h"
#include "sprites/InteractableSprites.h"
#include "sprites/IndicatorSprites.h"
#include "Enemy.h"
#include "Item.h"
#include "Mediator.h"
#include "Cell.h"

//interactable types
#define INTERACTABLE_TYPE_SWITCH 0
#define INTERACTABLE_TYPE_DOOR 1
#define INTERACTABLE_TYPE_DOOR_ACTIVATABLE 2
#define INTERACTABLE_TYPE_TRAP_DOOR 3
#define INTERACTABLE_TYPE_CARD_READER 4
#define INTERACTABLE_TYPE_SWITCH_ACTIVE 5
#define INTERACTABLE_TYPE_EXIT 6
#define INTERACTABLE_TYPE_CARD_READER_ACTIVE 7
#define INTERACTABLE_TYPE_DOOR2 8
#define INTERACTABLE_TYPE_TRAP_DOOR2 9

//interactable states
#define INTERACTABLE_STATE_OFF 0
#define INTERACTABLE_STATE_ON 1

#define ACTIVATION_SOURCE_FITZROY 0
#define ACTIVATION_SOURCE_LINK 1

typedef struct Interactable Interactable;

struct Interactable
{
    Position position;
    u8 type : 4; //7 different types sound reasonable initially
    u8 state : 2; //0,1,2,3 off, on, other, other2 (not sure if I want these other options but I will leave the option for now)
    u8 spriteNumber : 3;
    u8 enemyLink : 4; //surely I wouldn't have more than 15 enemies
    u8 interactableLink : 4; //15 interactables sounds safe
    u8 visible : 1; //for visibility in particular doors
    u8 used : 1; //we want an interactable to only be used once (such as a switch) rather than allowing us to switched between two values continuously
    u8 broken : 1; //we want to determine if an interactable is broken or working, what this means is when broken the state cannot be changed
};

extern Interactable mapInteractables[MAX_INTERACTABLE];

extern u8 mapInteractableCount;

//draw interactable to screen
void drawInteractable(Interactable* interactable);

//activate the interactable and trigger the link
void activateInteractable(Interactable* interactable, u8 activationSource);

//clear the space that an interactable occupies from the screen
void clearInteractable(Interactable* interactable);

#endif