#include "Map.h"
#define NULL 0

//map arrays
u8* mapArray = NULL;

//Fitzroy position
i8 mapFitzroyXPosition = -1;
i8 mapFitzroyYPosition = -1;

//enemies
Enemy mapEnemies[MAX_ENEMY];

//interactables
Interactable mapInteractables[MAX_INTERACTABLE];

//items
Item mapItems[MAX_ITEM];

//map counts
u8 mapXTileCount = 0;
u8 mapInteractableCount = 0;
u8 mapItemCount = 0;
u8 mapEnemyCount = 0;

u8 mapRedraw = TRUE;



void initMap(u8 selectedLevel)
{
    chooseMap(selectedLevel);

    //7 = tiles, interactables and items visible
	//9 = death squares and items visible
    //15 = death, tiles, interactables, items
    //16 = tails
    //64 = tiles revealed on touch
    
    
    //default difficulty see everything
    hiddenFlag = 15;
    
    if(difficultyLevel == 2)
    {
        //death, items always visible, and tiles revealed on touch
        hiddenFlag = 73;
    }
    
    if(difficultyLevel == 3)
    {
        //death and tails
        hiddenFlag = 24;
    }


    mapRedraw = TRUE;
}

void drawMap()
{
	u8 i = 0;

    if(mapRedraw)
    {
        u8* mapMemory;        
        u8 j = 0;
        u16 position = 0;

        //position is the element in the map tile array
        for (i = 0; i < 20; ++i)
        {            
            for (j = 0; j < 12; ++j)
            {
                //dont draw empty cells
                if (mapCells[position].graphicType != GRAPHIC_TYPE_EMPTY)
                {
                    //only draw cells that are visible
                    if(mapCells[position].visible)
                    {
                        mapMemory = cpct_getScreenPtr(CPCT_VMEM_START, xPositions[i], yPositions[j]);
                        
                        switch(mapCells[position].graphicType)
                        {
                            case GRAPHIC_TYPE_TILE:
                            {
                                if((mapCells[position].death && (hiddenFlag & DRAW_DEATH)) || (hiddenFlag & DRAW_TILES))
                                {
                                    //draw the tile graphic
                                    cpct_drawSprite(tileGraphicSet[mapCells[position].graphicNumber.tile], mapMemory, TILE_WIDTH_BYTES, TILE_HEIGHT);    
                                }

                                //always draw the tile broken even if the tile itself isn't revealed to the player
                                if(mapCells[position].broken)
                                {
                                    cpct_drawSpriteMasked (G_cracked_tile_indicator, mapMemory, TILE_WIDTH_BYTES, TILE_HEIGHT);
                                }
                            }
                            break;
         
                            case GRAPHIC_TYPE_ITEM:
                            {
                                if(hiddenFlag & DRAW_ITEMS)
                                {
                                    //an item should not be drawn once it is picked up or used in this case
                                    if(mapCells[position].used == FALSE)
                                    {
                                        //draw item graphic
                                        cpct_drawSprite(itemGraphicSet[mapCells[position].graphicNumber.item], mapMemory, TILE_WIDTH_BYTES, TILE_HEIGHT);
                                    }
                                }                           
                            }
                            break;
        
                            case GRAPHIC_TYPE_INTERACTABLE:
                            {
                                if(hiddenFlag & DRAW_INTERACTABLES)
                                {
                                    //draw interactable graphic
                                    cpct_drawSprite(interactableGraphicSet[mapCells[position].graphicNumber.interactable], mapMemory, TILE_WIDTH_BYTES, TILE_HEIGHT);
        
                                    //draw an x if the interactable is in a broken state 
                                    if(mapCells[position].broken)
                                    {
                                        cpct_drawSpriteMasked (G_broken_interactable_indicator, mapMemory, TILE_WIDTH_BYTES, TILE_HEIGHT);
                                    }
                                }
                            }
                            break;
                        }
                    }
                }

                ++position;
            }
        }
		
        mapRedraw = FALSE;
    }

    //draw the enemies on screen separately outside of the main redraw loop
    //this is out here because typically enemies are not static and contain more complicated logic then tiles, items and interactables
	for (i = 0; i<mapEnemyCount; ++i)
	{
		drawEnemy(&mapEnemies[i]);
	}
}

void chooseMap(u8 selectedLevel)
{

    switch(selectedLevel)
    {
        case MAP_A0:
        {
            mapXTileCount = A0_INTRO_X_TILE_COUNT;
            mapArray = map_a0_intro;
            mapFitzroyXPosition = A0_INTRO_FITZROY_X;
            mapFitzroyYPosition = A0_INTRO_FITZROY_Y;
        
            mapItemCount = 0;//A0_INTRO_ITEM_COUNT;
            mapInteractableCount = A0_INTRO_INTERACTABLE_COUNT;
            mapEnemyCount = A0_INTRO_ENEMY_COUNT;
        
            //load the level time limit
            levelTimerSeconds = A0_INTRO_TIME_LIMIT;
        
            //load the raw enemies from the map file into the array of Enemy objects
            loadEnemies(enemy_a0_intro);
        
            //load the raw interactables from the map file into the array of Interactable objects
            loadInteractables(interactable_a0_intro);
            
            //load the raw items from the map file into the array of Item objects
            //loadItems(item_a0_intro);
        
            initAllCells();
            initTileCells(mapArray);
            //initItemCells(item_a0_intro, mapItemCount, ITEM_COMPONENT_COUNT);
            initInteractableCells(interactable_a0_intro, mapInteractableCount, INTERACTABLE_COMPONENT_COUNT);
        }
        break;
        
        case MAP_A1:
        {
            mapXTileCount = A1_KEYS_X_TILE_COUNT;
            mapArray = map_a1_keys;
            mapFitzroyXPosition = A1_KEYS_FITZROY_X;
            mapFitzroyYPosition = A1_KEYS_FITZROY_Y;
        
            mapItemCount = A1_KEYS_ITEM_COUNT;
            mapInteractableCount = A1_KEYS_INTERACTABLE_COUNT;
            mapEnemyCount = A1_KEYS_ENEMY_COUNT;
        
            //load the level time limit
            levelTimerSeconds = A1_KEYS_TIME_LIMIT;
        
            //load the raw enemies from the map file into the array of Enemy objects
            loadEnemies(enemy_a1_keys);
        
            //load the raw interactables from the map file into the array of Interactable objects
            loadInteractables(interactable_a1_keys);
            
            //load the raw items from the map file into the array of Item objects
            loadItems(item_a1_keys);
        
            initAllCells();
            initTileCells(mapArray);
            initItemCells(item_a1_keys, mapItemCount, ITEM_COMPONENT_COUNT);
            initInteractableCells(interactable_a1_keys, mapInteractableCount, INTERACTABLE_COMPONENT_COUNT);
        
        }
        break;
        
        case MAP_A2:
        {
            mapXTileCount = A2_PLATFORMS_X_TILE_COUNT;
            mapArray = map_a2_platforms;
            mapFitzroyXPosition = A2_PLATFORMS_FITZROY_X;
            mapFitzroyYPosition = A2_PLATFORMS_FITZROY_Y;
        
            mapItemCount = 0;//A2_PLATFORMS_ITEM_COUNT;
            mapInteractableCount = A2_PLATFORMS_INTERACTABLE_COUNT;
            mapEnemyCount = A2_PLATFORMS_ENEMY_COUNT;
        
            //load the level time limit
            levelTimerSeconds = A2_PLATFORMS_TIME_LIMIT;
        
            //load the raw enemies from the map file into the array of Enemy objects
            loadEnemies(enemy_a2_platforms);
        
            //load the raw interactables from the map file into the array of Interactable objects
            loadInteractables(interactable_a2_platforms);
            
            //load the raw items from the map file into the array of Item objects
            //loadItems(item_a2_platforms);
        
            initAllCells();
            initTileCells(mapArray);
            //initItemCells(item_a2_platforms, mapItemCount, ITEM_COMPONENT_COUNT);
            initInteractableCells(interactable_a2_platforms, mapInteractableCount, INTERACTABLE_COMPONENT_COUNT);
        
        }
        break;
        
        case MAP_A3:
        {
            mapXTileCount = A3_DEATH_X_TILE_COUNT;
            mapArray = map_a3_death;
            mapFitzroyXPosition = A3_DEATH_FITZROY_X;
            mapFitzroyYPosition = A3_DEATH_FITZROY_Y;
        
            mapItemCount = 0;//A3_DEATH_ITEM_COUNT;
            mapInteractableCount = A3_DEATH_INTERACTABLE_COUNT;
            mapEnemyCount = A3_DEATH_ENEMY_COUNT;
        
            //load the level time limit
            levelTimerSeconds = A3_DEATH_TIME_LIMIT;
        
            //load the raw enemies from the map file into the array of Enemy objects
            loadEnemies(enemy_a3_death);
        
            //load the raw interactables from the map file into the array of Interactable objects
            loadInteractables(interactable_a3_death);
            
            //load the raw items from the map file into the array of Item objects
            //loadItems(item_a3_death);
        
            initAllCells();
            initTileCells(mapArray);
            //initItemCells(item_a3_death, mapItemCount, ITEM_COMPONENT_COUNT);
            initInteractableCells(interactable_a3_death, mapInteractableCount, INTERACTABLE_COMPONENT_COUNT);
        
        }
        break;
        
        case MAP_A4:
        {
            mapXTileCount = A4_LASERS_X_TILE_COUNT;
            mapArray = map_a4_lasers;
            mapFitzroyXPosition = A4_LASERS_FITZROY_X;
            mapFitzroyYPosition = A4_LASERS_FITZROY_Y;
        
            mapItemCount = 0;//A4_LASERS_ITEM_COUNT;
            mapInteractableCount = A4_LASERS_INTERACTABLE_COUNT;
            mapEnemyCount = A4_LASERS_ENEMY_COUNT;
        
            //load the level time limit
            levelTimerSeconds = A4_LASERS_TIME_LIMIT;
        
            //load the raw enemies from the map file into the array of Enemy objects
            loadEnemies(enemy_a4_lasers);
        
            //load the raw interactables from the map file into the array of Interactable objects
            loadInteractables(interactable_a4_lasers);
            
            //load the raw items from the map file into the array of Item objects
            //loadItems(item_a4_lasers);
        
            initAllCells();
            initTileCells(mapArray);
            //initItemCells(item_a4_lasers, mapItemCount, ITEM_COMPONENT_COUNT);
            initInteractableCells(interactable_a4_lasers, mapInteractableCount, INTERACTABLE_COMPONENT_COUNT);
        
        }
        break;
        
        case MAP_A5:
        {
            mapXTileCount = A5_CRACKED_TILES_X_TILE_COUNT;
            mapArray = map_a5_cracked_tiles;
            mapFitzroyXPosition = A5_CRACKED_TILES_FITZROY_X;
            mapFitzroyYPosition = A5_CRACKED_TILES_FITZROY_Y;
        
            mapItemCount = A5_CRACKED_TILES_ITEM_COUNT;
            mapInteractableCount = A5_CRACKED_TILES_INTERACTABLE_COUNT;
            mapEnemyCount = 0;//A5_CRACKED_TILES_ENEMY_COUNT;
        
            //load the level time limit
            levelTimerSeconds = A5_CRACKED_TILES_TIME_LIMIT;
        
            //load the raw enemies from the map file into the array of Enemy objects
            //loadEnemies(enemy_a5_cracked_tiles);
        
            //load the raw interactables from the map file into the array of Interactable objects
            loadInteractables(interactable_a5_cracked_tiles);
            
            //load the raw items from the map file into the array of Item objects
            loadItems(item_a5_cracked_tiles);
        
            initAllCells();
            initTileCells(mapArray);
            initItemCells(item_a5_cracked_tiles, mapItemCount, ITEM_COMPONENT_COUNT);
            initInteractableCells(interactable_a5_cracked_tiles, mapInteractableCount, INTERACTABLE_COMPONENT_COUNT);
        
        }
        break;
        
        case MAP_B6:
        {
            mapXTileCount = B6_TRAPDOORS_X_TILE_COUNT;
            mapArray = map_b6_trapdoors;
            mapFitzroyXPosition = B6_TRAPDOORS_FITZROY_X;
            mapFitzroyYPosition = B6_TRAPDOORS_FITZROY_Y;
        
            mapItemCount = B6_TRAPDOORS_ITEM_COUNT;
            mapInteractableCount = B6_TRAPDOORS_INTERACTABLE_COUNT;
            mapEnemyCount = B6_TRAPDOORS_ENEMY_COUNT;
        
            //load the level time limit
            levelTimerSeconds = B6_TRAPDOORS_TIME_LIMIT;
        
            //load the raw enemies from the map file into the array of Enemy objects
            loadEnemies(enemy_b6_trapdoors);
        
            //load the raw interactables from the map file into the array of Interactable objects
            loadInteractables(interactable_b6_trapdoors);
            
            //load the raw items from the map file into the array of Item objects
            loadItems(item_b6_trapdoors);
        
            initAllCells();
            initTileCells(mapArray);
            initItemCells(item_b6_trapdoors, mapItemCount, ITEM_COMPONENT_COUNT);
            initInteractableCells(interactable_b6_trapdoors, mapInteractableCount, INTERACTABLE_COMPONENT_COUNT);
        
        }
        break;
        
        case MAP_B7:
        {
            mapXTileCount = B7_SHOES_X_TILE_COUNT;
            mapArray = map_b7_shoes;
            mapFitzroyXPosition = B7_SHOES_FITZROY_X;
            mapFitzroyYPosition = B7_SHOES_FITZROY_Y;
        
            mapItemCount = B7_SHOES_ITEM_COUNT;
            mapInteractableCount = B7_SHOES_INTERACTABLE_COUNT;
            mapEnemyCount = B7_SHOES_ENEMY_COUNT;
        
            //load the level time limit
            levelTimerSeconds = B7_SHOES_TIME_LIMIT;
        
            //load the raw enemies from the map file into the array of Enemy objects
            loadEnemies(enemy_b7_shoes);
        
            //load the raw interactables from the map file into the array of Interactable objects
            loadInteractables(interactable_b7_shoes);
            
            //load the raw items from the map file into the array of Item objects
            loadItems(item_b7_shoes);
        
            initAllCells();
            initTileCells(mapArray);
            initItemCells(item_b7_shoes, mapItemCount, ITEM_COMPONENT_COUNT);
            initInteractableCells(interactable_b7_shoes, mapInteractableCount, INTERACTABLE_COMPONENT_COUNT);
        
        }
        break;
        
        case MAP_B8:
        {
            mapXTileCount = B8_MAGNIFYING_GLASS_X_TILE_COUNT;
            mapArray = map_b8_magnifying_glass;
            mapFitzroyXPosition = B8_MAGNIFYING_GLASS_FITZROY_X;
            mapFitzroyYPosition = B8_MAGNIFYING_GLASS_FITZROY_Y;
        
            mapItemCount = B8_MAGNIFYING_GLASS_ITEM_COUNT;
            mapInteractableCount = B8_MAGNIFYING_GLASS_INTERACTABLE_COUNT;
            mapEnemyCount = B8_MAGNIFYING_GLASS_ENEMY_COUNT;
        
            //load the level time limit
            levelTimerSeconds = B8_MAGNIFYING_GLASS_TIME_LIMIT;
        
            //load the raw enemies from the map file into the array of Enemy objects
            loadEnemies(enemy_b8_magnifying_glass);
        
            //load the raw interactables from the map file into the array of Interactable objects
            loadInteractables(interactable_b8_magnifying_glass);
            
            //load the raw items from the map file into the array of Item objects
            loadItems(item_b8_magnifying_glass);
        
            initAllCells();
            initTileCells(mapArray);
            initItemCells(item_b8_magnifying_glass, mapItemCount, ITEM_COMPONENT_COUNT);
            initInteractableCells(interactable_b8_magnifying_glass, mapInteractableCount, INTERACTABLE_COMPONENT_COUNT);
        
        }
        break;
        
        case MAP_B9:
        {
            mapXTileCount = B9_SWITCHED_DOORS_X_TILE_COUNT;
            mapArray = map_b9_switched_doors;
            mapFitzroyXPosition = B9_SWITCHED_DOORS_FITZROY_X;
            mapFitzroyYPosition = B9_SWITCHED_DOORS_FITZROY_Y;
        
            mapItemCount = 0;//B9_SWITCHED_DOORS_ITEM_COUNT;
            mapInteractableCount = B9_SWITCHED_DOORS_INTERACTABLE_COUNT;
            mapEnemyCount = B9_SWITCHED_DOORS_ENEMY_COUNT;
        
            //load the level time limit
            levelTimerSeconds = B9_SWITCHED_DOORS_TIME_LIMIT;
        
            //load the raw enemies from the map file into the array of Enemy objects
            loadEnemies(enemy_b9_switched_doors);
        
            //load the raw interactables from the map file into the array of Interactable objects
            loadInteractables(interactable_b9_switched_doors);
            
            //load the raw items from the map file into the array of Item objects
            //loadItems(item_b9_switched_doors);
        
            initAllCells();
            initTileCells(mapArray);
            //initItemCells(item_b9_switched_doors, mapItemCount, ITEM_COMPONENT_COUNT);
            initInteractableCells(interactable_b9_switched_doors, mapInteractableCount, INTERACTABLE_COMPONENT_COUNT);
        }
        break;
        
        case MAP_B10:
        {
            mapXTileCount = B10_SPANNERS_X_TILE_COUNT;
            mapArray = map_b10_spanners;
            mapFitzroyXPosition = B10_SPANNERS_FITZROY_X;
            mapFitzroyYPosition = B10_SPANNERS_FITZROY_Y;
        
            mapItemCount = B10_SPANNERS_ITEM_COUNT;
            mapInteractableCount = B10_SPANNERS_INTERACTABLE_COUNT;
            mapEnemyCount = B10_SPANNERS_ENEMY_COUNT;
        
            //load the level time limit
            levelTimerSeconds = B10_SPANNERS_TIME_LIMIT;
        
            //load the raw enemies from the map file into the array of Enemy objects
            loadEnemies(enemy_b10_spanners);
        
            //load the raw interactables from the map file into the array of Interactable objects
            loadInteractables(interactable_b10_spanners);
            
            //load the raw items from the map file into the array of Item objects
            loadItems(item_b10_spanners);
        
            initAllCells();
            initTileCells(mapArray);
            initItemCells(item_b10_spanners, mapItemCount, ITEM_COMPONENT_COUNT);
            initInteractableCells(interactable_b10_spanners, mapInteractableCount, INTERACTABLE_COMPONENT_COUNT);
        }
        break;

        case MAP_C11:
        {
            mapXTileCount = C11_X_TILE_COUNT;
            mapArray = map_c11;
            mapFitzroyXPosition = C11_FITZROY_X;
            mapFitzroyYPosition = C11_FITZROY_Y;
        
            mapItemCount = C11_ITEM_COUNT;
            mapInteractableCount = C11_INTERACTABLE_COUNT;
            mapEnemyCount = C11_ENEMY_COUNT;
        
            //load the level time limit
            levelTimerSeconds = C11_TIME_LIMIT;
        
            //load the raw enemies from the map file into the array of Enemy objects
            loadEnemies(enemy_c11);
        
            //load the raw interactables from the map file into the array of Interactable objects
            loadInteractables(interactable_c11);
            
            //load the raw items from the map file into the array of Item objects
            loadItems(item_c11);
        
            initAllCells();
            initTileCells(mapArray);
            initItemCells(item_c11, mapItemCount, ITEM_COMPONENT_COUNT);
            initInteractableCells(interactable_c11, mapInteractableCount, INTERACTABLE_COMPONENT_COUNT);
        }
        break;

        case MAP_C12:
        {
            mapXTileCount = C12_X_TILE_COUNT;
            mapArray = map_c12;
            mapFitzroyXPosition = C12_FITZROY_X;
            mapFitzroyYPosition = C12_FITZROY_Y;
        
            mapItemCount = C12_ITEM_COUNT;
            mapInteractableCount = C12_INTERACTABLE_COUNT;
            mapEnemyCount = C12_ENEMY_COUNT;
        
            //load the level time limit
            levelTimerSeconds = C12_TIME_LIMIT;
        
            //load the raw enemies from the map file into the array of Enemy objects
            loadEnemies(enemy_c12);
        
            //load the raw interactables from the map file into the array of Interactable objects
            loadInteractables(interactable_c12);
            
            //load the raw items from the map file into the array of Item objects
            loadItems(item_c12);
        
            initAllCells();
            initTileCells(mapArray);
            initItemCells(item_c12, mapItemCount, ITEM_COMPONENT_COUNT);
            initInteractableCells(interactable_c12, mapInteractableCount, INTERACTABLE_COMPONENT_COUNT);
        }
        break;
    } 
}

void destroyMap()
{
    mapArray = NULL;
}

void loadInteractables(const u8* rawMapInteractables)
{
    u8 i = 0;

    for(i = 0; i<mapInteractableCount*INTERACTABLE_COMPONENT_COUNT; i+=INTERACTABLE_COMPONENT_COUNT)
    {
        //the index of the interactable in the interactable array
        u8 interactableIndex = i / INTERACTABLE_COMPONENT_COUNT;
		u8 type = 0;

		mapInteractables[interactableIndex].position.x = rawMapInteractables[i];
		mapInteractables[interactableIndex].position.y = rawMapInteractables[i+1];
		mapInteractables[interactableIndex].type = rawMapInteractables[i+2];
        mapInteractables[interactableIndex].enemyLink = rawMapInteractables[i+3];
        mapInteractables[interactableIndex].interactableLink = rawMapInteractables[i+4];
        mapInteractables[interactableIndex].broken = rawMapInteractables[i+5];
        mapInteractables[interactableIndex].visible = TRUE; //default is visible
		mapInteractables[interactableIndex].used = FALSE; //default is unused

		type = mapInteractables[interactableIndex].type;

		//active switch is on
		//inactive switch is off
		//cardreader is on
		//activatable door is link dependent
		//door is on
		//trapdoor is on

        //switch (0) is off by default, all other interactable types are on
		switch (type)
		{
			case INTERACTABLE_TYPE_TRAP_DOOR:
			case INTERACTABLE_TYPE_DOOR:
				mapInteractables[interactableIndex].state = INTERACTABLE_STATE_ON;
			break;

			case INTERACTABLE_TYPE_CARD_READER:
			case INTERACTABLE_TYPE_SWITCH:
				mapInteractables[interactableIndex].state = INTERACTABLE_STATE_OFF;

				//update the linked interactable with the correct state (currently only applies to activatable doors)
				if (mapInteractables[interactableIndex].interactableLink != UNDEFINED_LINK)
				{
					mapInteractables[mapInteractables[interactableIndex].interactableLink].state = INTERACTABLE_STATE_OFF;
					if (mapInteractables[mapInteractables[interactableIndex].interactableLink].type == INTERACTABLE_TYPE_DOOR_ACTIVATABLE)
					{
						mapInteractables[mapInteractables[interactableIndex].interactableLink].visible = FALSE;
					}
				}
				
			break;
			
			case INTERACTABLE_TYPE_CARD_READER_ACTIVE:
			case INTERACTABLE_TYPE_SWITCH_ACTIVE:
				mapInteractables[interactableIndex].state = INTERACTABLE_STATE_ON;

				//update the linked interactable with the correct state (currently only applies to activatable doors)
				if (mapInteractables[interactableIndex].interactableLink != UNDEFINED_LINK)
				{
					mapInteractables[mapInteractables[interactableIndex].interactableLink].state = INTERACTABLE_STATE_ON;
					if (mapInteractables[mapInteractables[interactableIndex].interactableLink].type == INTERACTABLE_TYPE_DOOR_ACTIVATABLE)
					{
						mapInteractables[mapInteractables[interactableIndex].interactableLink].visible = TRUE;
					}
				}
			break;

			case INTERACTABLE_TYPE_DOOR_ACTIVATABLE:
				//do nothing (this state should be effected by the state update of the linked interactable)
			break;			
		}

		//update the linked enemy state to match the interactable
		if (mapInteractables[interactableIndex].enemyLink != UNDEFINED_LINK)
		{
			mapEnemies[mapInteractables[interactableIndex].enemyLink].state = mapInteractables[interactableIndex].state;
		}
			        
        //todo handle graphics and interactable types correctly
		mapInteractables[interactableIndex].spriteNumber = mapInteractables[i].type;
    }
}

void loadEnemies(const u8* rawMapEnemies)
{
    u8 i = 0;

    for(i = 0; i<mapEnemyCount*ENEMY_COMPONENT_COUNT; i+=ENEMY_COMPONENT_COUNT)
    {
        //the index of the enemy in the enemy array
		u8 enemyIndex = i / ENEMY_COMPONENT_COUNT;
        u8 projectileEnd;
        
        mapEnemies[enemyIndex].state = ENEMY_STATE_ACTIVE;
		mapEnemies[enemyIndex].object.position.x = rawMapEnemies[i] * TILE_WIDTH_BYTES;
		mapEnemies[enemyIndex].object.position.y = rawMapEnemies[i+1] * TILE_HEIGHT;
		mapEnemies[enemyIndex].previousPosition.x = mapEnemies[enemyIndex].object.position.x;
		mapEnemies[enemyIndex].previousPosition.y = mapEnemies[enemyIndex].object.position.y;
		mapEnemies[enemyIndex].object.velocity.y = 0;
		mapEnemies[enemyIndex].object.velocity.x = 0;
		mapEnemies[enemyIndex].object.height = TILE_HEIGHT;
		mapEnemies[enemyIndex].object.width = TILE_WIDTH_BYTES;
		mapEnemies[enemyIndex].type = rawMapEnemies[i+2];
        projectileEnd = rawMapEnemies[i+3] * TILE_HEIGHT;

		//todo handle graphics and enemy types correctly
        mapEnemies[enemyIndex].spriteNumber = mapEnemies[enemyIndex].type;        
        
        switch(mapEnemies[enemyIndex].type)
        {
            case ENEMY_TYPE_DOWN_LASER:
            {
				//the -TILE_HEIGHT is so that we don't include the size of the enemy in the length of the beam
				u8 beamLength = (projectileEnd - mapEnemies[enemyIndex].object.position.y - TILE_HEIGHT);
				initProjectile(&mapEnemies[enemyIndex].projectile, &mapEnemies[enemyIndex].object, PROJECTILE_WIDTH, beamLength, projectileEnd, FALSE);
            }
            break;

            case ENEMY_TYPE_DOWN_DROPPER:
            {
				initProjectile(&mapEnemies[enemyIndex].projectile, &mapEnemies[enemyIndex].object, PROJECTILE_WIDTH, DROPPER_PROJECTILE_HEIGHT, projectileEnd, TRUE);
            }
            break;

            case ENEMY_TYPE_UP_DOWN2:
            case ENEMY_TYPE_UP_DOWN:
                mapEnemies[enemyIndex].direction = ENEMY_UP;
                mapEnemies[enemyIndex].object.velocity.y = ENEMY_UP;

                //enemies without projectiles should have the projectile set to non existant
                mapEnemies[enemyIndex].projectile.exists = FALSE;
            break;

			case ENEMY_TYPE_LEFT_RIGHT2:
            case ENEMY_TYPE_LEFT_RIGHT:
                mapEnemies[enemyIndex].direction = ENEMY_LEFT;
                mapEnemies[enemyIndex].object.velocity.x = ENEMY_LEFT;
			
				//drop down into default
            default:
            {
                //enemies without projectiles should have the projectile set to non existant
                mapEnemies[enemyIndex].projectile.exists = FALSE;
            }
            break;
        }
    }
}

void loadItems(const u8* rawMapItems)
{
    u8 i = 0;

    for(i = 0; i<mapItemCount*ITEM_COMPONENT_COUNT; i+=ITEM_COMPONENT_COUNT)
    {
        //the index of the item in the item array
		u8 itemIndex = i / ITEM_COMPONENT_COUNT;
		
		mapItems[itemIndex].position.x = rawMapItems[i];
		mapItems[itemIndex].position.y = rawMapItems[i+1];
        mapItems[itemIndex].type = rawMapItems[i+2];
        
        //initially items should not be in the inventory
        mapItems[itemIndex].inInventory = FALSE;

		//todo handle graphics and item types correctly
		mapItems[itemIndex].spriteNumber = mapItems[itemIndex].type;	
    }
}

void logicMap()
{
    u8 i = 0;

    //logic enemies
    for(i = 0; i<mapEnemyCount; ++i)
    {
        logicEnemy(&mapEnemies[i]);
    }
}