#include "MapManager.h"

const u8 gameOverText_[10] = {"Game Over\0"};
const u8 newHighScore_[15] = {"New High Score\0"};
const u8 victoriousText_[11] = {"Victorious\0"};

const u8 mapList_[MAP_COUNT] = {MAP_A0, MAP_A1, MAP_A2, MAP_A3, MAP_A4, MAP_A5, MAP_B6, MAP_B7, MAP_B8, MAP_B9, MAP_B10, MAP_C11, MAP_C12};

#define ASCII_A_ 65
#define ASCII_Z_ 90

#define START_LEVEL 0

u8* memory;

void startMapManager()
{
    //set the score to 0 before starting the game
    gameHud.gameScore = 0;

    //start from first level (that is the level at the beginning of the array)
    startLevel(START_LEVEL);
}

void startLevel(u8 level)
{
    initGame(mapList_[level]);    
    startGame();

    switch(gameResult)
    {
        case WIN:
        {
            //goto next level
            nextLevel(level);
        }
        break;
        
        case LOSS:
        {
            //goto loss screen
            endScreen(FALSE);
        }
        break;

        case QUIT:
        {
            //stop the level timer
            stopLevelTimer();

            //goto loss screen
            endScreen(FALSE);
        }
        break;
    }
}

void nextLevel(u8 currentLevel)
{
    if(currentLevel + 1 < MAP_COUNT)
    {
        //go onto the next level
        startLevel(currentLevel + 1);
    }
    else
    {
        //finished all the levels which means victory
        endScreen(TRUE);
    }
}

void endScreen(const u8 win)
{
    memory = cpct_getScreenPtr(CPCT_VMEM_START, 20, 20);   

    //stop the level timer so it can't be drawn anymore
    levelTimerActive = FALSE;

    //clear the screen
    cpct_clearScreen(0);

    if(win)
    {
        //draw victorious text
        cpct_drawStringM0(victoriousText_, memory, 1, 5);
    }
    else
    {
        //draw game over text
        cpct_drawStringM0(gameOverText_, memory, 1, 5);
    }

    //wait for user to press any key before going back to menu
    while(1)
    {
        cpct_scanKeyboard();

        if(cpct_isKeyPressed(Key_Return))
        {
            break;
        }
    }

    //update the highscore following the end of the game
    if(isHighScore())
    {
        enterYourScoreName();
        updateHighScore();
    }
}

void enterYourScoreName()
{    
    
    u8 letterCount = 0;
    u8 nextKey = ASCII_A_;
    u8 scoreUpPressed = FALSE;
    u8 scoreDownPressed = FALSE;
    u8 scoreEnterPressed = FALSE;
    memory = cpct_getScreenPtr(CPCT_VMEM_START, 20, 20);   

    //clear the screen
    cpct_clearScreen(0);

    //draw game over text
    cpct_drawStringM0(newHighScore_, memory, 1, 5);

    //draw the player score in the screen
    memory = cpct_getScreenPtr(CPCT_VMEM_START, 20, 40);   
    cpct_drawStringM0(scoreHud, memory, 1, 5);

    memcpy(newScoreName, "A..\0", 4);

    //choose the players initials using up/down/enter
    while(1)
    {   
        cpct_scanKeyboard();
        
        memory = cpct_getScreenPtr(CPCT_VMEM_START, 20, 50);   
        cpct_drawStringM0(newScoreName, memory, 1, 5);

        if(cpct_isKeyPressed(Key_CursorDown))
        {
            scoreUpPressed = TRUE;
        }
        else if(cpct_isKeyPressed(Key_CursorUp))
        {
            scoreDownPressed = TRUE;
        }
        else if(cpct_isKeyPressed(Key_Return))
        {
            scoreEnterPressed = TRUE;
        }
        else if(!cpct_isAnyKeyPressed())
        {
            if(scoreUpPressed)
            {
                //change letter on pressing up
                if(nextKey >= ASCII_Z_)
                {
                    //do nothing
                }
                else
                {
                    ++nextKey;
                    newScoreName[letterCount] = nextKey;
                }

                scoreUpPressed = FALSE;
            }
            else if(scoreDownPressed)
            {            
                //change letter on pressing down
                if(nextKey <= ASCII_A_)
                {
                    //do nothing
                }
                else
                {
                    --nextKey;
                    newScoreName[letterCount] = nextKey;
                }

                scoreDownPressed = FALSE;
            }
            else if(scoreEnterPressed)
            {
                //progress to the next letter
                ++letterCount;

                //reset to A
                nextKey = ASCII_A_;

                scoreEnterPressed = FALSE;

                //change the next . to A to make it clear that we are changing to the next letter
                if(letterCount < 3)
                {
                    newScoreName[letterCount] = nextKey;
                }

            }

            if(letterCount >= 3)
            {
                break;
            }
        }
    }  
}