#include "Projectile.h"

void initProjectile(Projectile* projectile, Object* enemyObject, u8 width, u8 height, u8 projectileEnd, u8 moves)
{
	projectile->object.width = width;
	projectile->object.height = height;
	projectile->object.position.x = enemyObject->position.x + 1;
	projectile->object.position.y = (enemyObject->position.y + TILE_HEIGHT); //projectile y position should start after the enemy
	projectile->previousPosition.x = projectile->object.position.x;
	projectile->previousPosition.y = projectile->object.position.y;
	projectile->object.velocity.y = 1;
	projectile->end = projectileEnd;
	projectile->startY = projectile->object.position.y;
	projectile->exists = 1;
	projectile->moves = moves;
}

void drawProjectile(Projectile* projectile)
{
	u8* memory;

	//projectile shouldn't be drawn if it has been "destroyed"
	if(projectile->exists)
	{
		//draw over the previous tail
		if (projectile->previousPosition.x != projectile->object.position.x || projectile->previousPosition.y != projectile->object.position.y)
		{
			//cover the previous position
			memory = cpct_getScreenPtr(CPCT_VMEM_START, projectile->previousPosition.x, projectile->previousPosition.y);
			cpct_drawSolidBox(memory, backgroundColour, projectile->object.width, projectile->object.height);
		}

		//draw a square to represent the projectile
		memory = cpct_getScreenPtr(CPCT_VMEM_START, projectile->object.position.x, projectile->object.position.y);
		cpct_drawSolidBox(memory, cpct_px2byteM0(3, 3), projectile->object.width, projectile->object.height);
	}
}

void logicProjectile(Projectile* projectile)
{
	//logic only needs to be run if the projectile actually moves down the screen
	if(projectile->moves)
	{
		//set previous position for the projectile
		projectile->previousPosition.x = projectile->object.position.x;
		projectile->previousPosition.y = projectile->object.position.y;

		//calculate new projectile y position from y velocity
		if(projectile->object.velocity.y > 0)
		{
			if(projectile->object.position.y + projectile->object.height < projectile->end)
			{
				projectile->object.position.y += 1;
			}
			else
			{
				//reached the end so the projectile should go back to the start
				projectile->object.position.y = projectile->startY;
			}             
		}
	}
}

void clearProjectile(Projectile* projectile)
{
	//cover the current position of the projectile
	u8* memory = cpct_getScreenPtr(CPCT_VMEM_START, projectile->object.position.x, projectile->object.position.y);
	cpct_drawSolidBox(memory, backgroundColour, projectile->object.width, projectile->object.height);
}