#include "OptionsMenu.h"

#define OPTIONS_MENU_TITLE_TEXT_LENGTH_ 8
#define DIFFICULTY_ITEM_TEXT_LENGTH_ 13
#define EXIT_OPTIONS_ITEM_TEXT_LENGTH_ 7

const u8 OPTIONS_MENU_TITLE_TEXT_[OPTIONS_MENU_TITLE_TEXT_LENGTH_] = "Options\0";
#define DIFFICULTY_ITEM_TEXT_ "Difficulty-1\0"
const u8 EXIT_OPTIONS_ITEM_TEXT_[EXIT_OPTIONS_ITEM_TEXT_LENGTH_] = "-Exit-\0";

#define CONTROLS_JOYSTICK_ITEM_TEXT_LENGTH_ 18
#define CONTROLS_CURSOR_ITEM_TEXT_LENGTH_ 16
#define CONTROLS_QAOP_ITEM_TEXT_LENGTH_ 14
#define CONTROLS_CURSOR2_ITEM_TEXT_LENGTH_ 17

const u8 CONTROLS_JOYSTICK_ITEM_TEXT_[CONTROLS_JOYSTICK_ITEM_TEXT_LENGTH_] = "Controls-Joystick\0";
const u8 CONTROLS_CURSOR_ITEM_TEXT_[CONTROLS_CURSOR_ITEM_TEXT_LENGTH_] = "Controls-Cursor\0";
const u8 CONTROLS_QAOP_ITEM_TEXT_[CONTROLS_QAOP_ITEM_TEXT_LENGTH_] = "Controls-QAOP\0";
const u8 CONTROLS_CURSOR2_ITEM_TEXT_[CONTROLS_CURSOR2_ITEM_TEXT_LENGTH_] = "Controls-Cursor2\0";

#define DIFFICULTY_LEVEL_TEXT_POSITION_ 11
#define ASCII_TO_NUMBER_OFFSET_ 48

Menu optionsMenu;
MenuItem optionsMenuItems[OPTIONS_MENUITEM_COUNT];
MenuItem controlsMenuItem_;
MenuItem difficultyMenuItem_;
MenuItem exitOptionsMenuItem_;

void interactControlsMenuItem_();
void interactDifficultyMenuItem_();
void exitOptionsMenu_();

void buildOptionsMenu()
{
    //setup menu items
    initMenuItem(&controlsMenuItem_);
    initMenuItem(&difficultyMenuItem_);
    initMenuItem(&exitOptionsMenuItem_);

    //set menu item text
    memcpy(controlsMenuItem_.itemText, CONTROLS_CURSOR_ITEM_TEXT_, sizeof(u8)*CONTROLS_CURSOR_ITEM_TEXT_LENGTH_);
    memcpy(difficultyMenuItem_.itemText, DIFFICULTY_ITEM_TEXT_, sizeof(u8)*DIFFICULTY_ITEM_TEXT_LENGTH_);
    memcpy(exitOptionsMenuItem_.itemText, EXIT_OPTIONS_ITEM_TEXT_, sizeof(u8)*EXIT_OPTIONS_ITEM_TEXT_LENGTH_);

    //set menu item run functions
    controlsMenuItem_.runMenuItem = interactControlsMenuItem_;
    difficultyMenuItem_.runMenuItem = interactDifficultyMenuItem_;
    exitOptionsMenuItem_.runMenuItem = exitOptionsMenu_;

    //add menu items to array
    memcpy(&optionsMenuItems[0], &controlsMenuItem_, sizeof(MenuItem));
    memcpy(&optionsMenuItems[1], &difficultyMenuItem_, sizeof(MenuItem));
    memcpy(&optionsMenuItems[2], &exitOptionsMenuItem_, sizeof(MenuItem));
    
    //set menu title
    memcpy(optionsMenu.menuTitle, OPTIONS_MENU_TITLE_TEXT_, sizeof(u8)*OPTIONS_MENU_TITLE_TEXT_LENGTH_);

    //add menuitems array to menu
    initMenu(&optionsMenu, optionsMenuItems, OPTIONS_MENUITEM_COUNT);
}

void runOptionsMenu()
{
    runMenu(&optionsMenu);
}

void exitOptionsMenu_()
{
    exitMenu(&optionsMenu);
}

void interactControlsMenuItem_()
{
    if(currentControlSchemeChoice + 1 < CONTROLS_SCHEME_CHOICE_COUNT)
    {
        ++currentControlSchemeChoice;                
    } 
    else
    {
        currentControlSchemeChoice = 0;
    }

    switch(currentControlSchemeChoice)
    {
        case 0:
        {            
            memcpy(optionsMenuItems[0].itemText, CONTROLS_CURSOR_ITEM_TEXT_, sizeof(u8)*CONTROLS_CURSOR_ITEM_TEXT_LENGTH_);           
            setCursorControls();
        }
        break;

        case 1:
        {                        
            memcpy(optionsMenuItems[0].itemText, CONTROLS_JOYSTICK_ITEM_TEXT_, sizeof(u8)*CONTROLS_JOYSTICK_ITEM_TEXT_LENGTH_);
            setJoystickControls();
        }
        break;

        case 2:
        {            
            memcpy(optionsMenuItems[0].itemText, CONTROLS_QAOP_ITEM_TEXT_, sizeof(u8)*CONTROLS_QAOP_ITEM_TEXT_LENGTH_);
            setQAOPControls();
        }
        break;

        case 3:
        {            
            memcpy(optionsMenuItems[0].itemText, CONTROLS_CURSOR2_ITEM_TEXT_, sizeof(u8)*CONTROLS_CURSOR2_ITEM_TEXT_LENGTH_);
            setCursorControlsCopy();
        }
        break;
    }
}

//adjust the difficulty level
void interactDifficultyMenuItem_()
{
    if(difficultyLevel < MAX_DIFFICULTY)
    {
        ++difficultyLevel;
    }
    else
    {
        difficultyLevel = MIN_DIFFICULTY;
    }

    //update the menu item
    optionsMenuItems[1].itemText[DIFFICULTY_LEVEL_TEXT_POSITION_] = difficultyLevel + ASCII_TO_NUMBER_OFFSET_;
}